@@ -1,4 +1,4 @@
-# Copyright (C) 2009  Kouhei Sutou <kou@clear-code.com>
+# Copyright (C) 2009-2010  Kouhei Sutou <kou@clear-code.com>
 #
 # This library is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -15,24 +15,40 @@
 
 module Milter
   class Client
-    def register(session_class)
+    def status_on_error
+      @status_on_error ||= :accept
+    end
+
+    def status_on_error=(status)
+      @status_on_error = status
+    end
+
+    def register(session_class, *new_arguments)
       signal_connect("connection-established") do |_client, context|
-        setup_session(context, session_class)
+        setup_session(context, session_class, new_arguments)
       end
     end
 
     private
-    def setup_session(context, session_class)
-      session = session_class.new
-      context.instance_variable_set("@session", session)
+    def setup_session(context, session_class, session_new_arguments)
+      session_context = ClientSessionContext.new(context)
+      session = session_class.new(session_context, *session_new_arguments)
 
       [:negotiate, :connect, :helo, :envelope_from, :envelope_recipient,
        :data, :unknown, :header, :end_of_header, :body, :end_of_message,
        :abort].each do |event|
         if session.respond_to?(event)
           context.signal_connect(event) do |_context, *args|
-            session_context = ClientSessionContext.new(_context)
-            session.send(event, session_context, *args)
+            begin
+              if event == :end_of_message
+                session.send(event)
+              else
+                session.send(event, *args)
+              end
+            rescue Exception
+              Milter::Logger.error($!)
+              session_context.status = status_on_error
+            end
             session_context.status
           end
         end
@@ -41,17 +57,66 @@ module Milter
   end
 
   class ClientSession
-    def negotiate(context, option, macros_requests)
+    def initialize(context)
+      @context = context
+    end
+
+    def negotiate(option, macros_requests)
       [:connect, :helo, :envelope_from, :envelope_recipient,
        :body, :unknown, [:header, :headers], :end_of_header,
-       :body].each do |method_name, step_name|
+      ].each do |method_name, step_name|
         if respond_to?(method_name)
           step_name ||= method_name
-          step = Milter::Step.const_get("NO_#{step_name.to_s.upcase}")
-          option.remove_action(step)
+          step = Milter::StepFlags.const_get("NO_#{step_name.to_s.upcase}")
+          option.remove_step(step)
         end
       end
-      context.status = :continue
+      unless need_header_value_with_leading_space?
+        option.remove_step(Milter::StepFlags::HEADER_VALUE_WITH_LEADING_SPACE)
+      end
+      option.remove_step(Milter::StepFlags::NO_REPLY_MASK)
+      @context.status = :continue
+    end
+
+    private
+    def need_header_value_with_leading_space?
+      false
+    end
+
+    def accept
+      @context.status = :accept
+    end
+
+    def discard
+      @context.status = :discard
+    end
+
+    def reject(options={})
+      code = options[:code]
+      extended_code = options[:extended_code]
+      reason = options[:reason]
+      if code
+	# TODO: validate parameters.
+        @context.set_reply(code, extended_code, reason)
+      else
+        @context.status = :reject
+      end
+    end
+
+    def temporary_failure(options={})
+      code = options[:code]
+      extended_code = options[:extended_code]
+      reason = options[:reason]
+      if code
+	# TODO: validate parameters.
+        @context.set_reply(code, extended_code, reason)
+      else
+        @context.status = :temporary_failure
+      end
+    end
+
+    def quarantine
+      @context.status = :quarantine
     end
   end
 
