@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009  Kouhei Sutou <kou@cozmixng.org>
+# Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
 #
 # This library is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -28,6 +28,12 @@ require 'milter_manager.so'
 require 'milter/compatible'
 
 require 'milter/manager/child-context'
+require 'milter/manager/connection-check-context'
+require 'milter/manager/netstat-connection-checker'
+
+require 'milter/manager/policy-manager'
+require 'milter/manager/address-matcher'
+require 'milter/manager/breaker'
 
 require 'milter/manager/debian-init-detector'
 require 'milter/manager/redhat-detector'
@@ -37,71 +43,7 @@ require 'milter/manager/pkgsrc-rc-detector'
 module Milter::Manager
   class Configuration
     def dump
-      result = ''
-
-      result << "package.platform = #{package_platform.inspect}\n"
-      result << "package.options = #{package_options.inspect}\n"
-      result << "\n"
-
-      result << "security.privilege_mode = #{privilege_mode?}\n"
-      result << "security.effective_user = #{effective_user.inspect}\n"
-      result << "security.effective_group = #{effective_group.inspect}\n"
-      result << "\n"
-
-      result << "manager.connection_spec = #{manager_connection_spec.inspect}\n"
-      result << "manager.unix_socket_mode = 0%o\n" % manager_unix_socket_mode
-      result << "manager.unix_socket_group = #{manager_unix_socket_group.inspect}\n"
-      result << "manager.remove_unix_socket_on_create = #{remove_manager_unix_socket_on_create?}\n"
-      result << "manager.remove_unix_socket_on_close = #{remove_manager_unix_socket_on_close?}\n"
-      result << "manager.daemon = #{daemon?}\n"
-      result << "manager.pid_file = #{pid_file.inspect}\n"
-      _maintenance_interval = maintenance_interval
-      _maintenance_interval = nil if _maintenance_interval.zero?
-      result << "manager.maintenance_interval = #{_maintenance_interval.inspect}\n"
-      result << "manager.suspend_time_on_unacceptable = #{suspend_time_on_unacceptable.inspect}\n"
-      result << "manager.max_connections = #{max_connections.inspect}\n"
-      result << "manager.max_file_descriptors = #{max_file_descriptors.inspect}\n"
-      result << "manager.custom_configuration_directory = #{custom_configuration_directory.inspect}\n"
-      result << "\n"
-
-      result << "controller.connection_spec = #{controller_connection_spec.inspect}\n"
-      result << "controller.unix_socket_mode = 0%o\n" % controller_unix_socket_mode
-      result << "controller.unix_socket_group = #{@controller_unix_socket_group.inspect}\n"
-      result << "controller.remove_unix_socket_on_create = #{remove_controller_unix_socket_on_create?}\n"
-      result << "controller.remove_unix_socket_on_close = #{remove_controller_unix_socket_on_close?}\n"
-      result << "\n"
-
-      applicable_conditions.each do |condition|
-        result << "define_applicable_condition(#{condition.name.inspect}) do |condition|\n"
-        result << "  condition.description = #{condition.description.inspect}\n"
-        result << "end\n"
-        result << "\n"
-      end
-
-      eggs.each do |egg|
-        result << "define_milter(#{egg.name.inspect}) do |milter|\n"
-        result << "  milter.connection_spec = #{egg.connection_spec.inspect}\n"
-        result << "  milter.description = #{egg.description.inspect}\n"
-        result << "  milter.enabled = #{egg.enabled?}\n"
-        fallback_status = egg.fallback_status.nick
-        result << "  milter.fallback_status = #{fallback_status.inspect}\n"
-        result << "  milter.evaluation_mode = #{egg.evaluation_mode?}\n"
-        condition_names = egg.applicable_conditions.collect do |condition|
-          condition.name
-        end
-        result << "  milter.applicable_conditions = #{condition_names.inspect}\n"
-        result << "  milter.command = #{egg.command.inspect}\n"
-        result << "  milter.command_options = #{egg.command_options.inspect}\n"
-        result << "  milter.user_name = #{egg.user_name.inspect}\n"
-        result << "  milter.connection_timeout = #{egg.connection_timeout}\n"
-        result << "  milter.writing_timeout = #{egg.writing_timeout}\n"
-        result << "  milter.reading_timeout = #{egg.reading_timeout}\n"
-        result << "  milter.end_of_message_timeout = #{egg.end_of_message_timeout}\n"
-        result << "end\n"
-        result << "\n"
-      end
-
-      result.strip
+      ConfigurationDumper.new(self).dump
     rescue Exception
       [$!.message, $@].join("\n")
     end
@@ -111,12 +53,173 @@ module Milter::Manager
       return nil if _package_options.nil?
 
       options = {}
-      _package_options.split(/,/).each do |option|
-        name, value = option.split(/=/, 2)
+      _package_options.split(/\s*,\s*/).each do |option|
+        name, value = option.split(/\s*=\s*/, 2)
         options[name] = value
       end
       options
     end
+
+    def maintained
+      maintained_hooks.each do |hook|
+        hook.call(self)
+      end
+    end
+
+    def maintained_hooks
+      @maintained_hooks ||= []
+    end
+
+    def clear
+      @maintained_hooks = nil
+    end
+  end
+
+  class ConfigurationDumper
+    def initialize(configuration)
+      @configuration = configuration
+      @locations = configuration.locations
+    end
+
+    def dump
+      @result = ''
+
+      c = @configuration
+      dump_item("package.platform", c.package_platform.inspect)
+      dump_item("package.options", c.package_options.inspect)
+      @result << "\n"
+
+      dump_item("security.privilege_mode", c.privilege_mode?)
+      dump_item("security.effective_user", c.effective_user.inspect)
+      dump_item("security.effective_group", c.effective_group.inspect)
+      @result << "\n"
+
+      dump_item("manager.connection_spec", c.manager_connection_spec.inspect)
+      dump_item("manager.unix_socket_mode", "0%o" % c.manager_unix_socket_mode)
+      dump_item("manager.unix_socket_group", c.manager_unix_socket_group.inspect)
+      dump_item("manager.remove_unix_socket_on_create",
+                c.remove_manager_unix_socket_on_create?)
+      dump_item("manager.remove_unix_socket_on_close",
+                c.remove_manager_unix_socket_on_close?)
+      dump_item("manager.daemon", c.daemon?)
+      dump_item("manager.pid_file", c.pid_file.inspect)
+      _maintenance_interval = c.maintenance_interval
+      _maintenance_interval = nil if _maintenance_interval.zero?
+      dump_item("manager.maintenance_interval", _maintenance_interval.inspect)
+      dump_item("manager.suspend_time_on_unacceptable",
+                c.suspend_time_on_unacceptable.inspect)
+      dump_item("manager.max_connections", c.max_connections.inspect)
+      dump_item("manager.max_file_descriptors", c.max_file_descriptors.inspect)
+      dump_item("manager.custom_configuration_directory",
+                c.custom_configuration_directory.inspect)
+      dump_item("manager.connection_check_interval",
+                c.connection_check_interval.inspect)
+      @result << "\n"
+
+      dump_item("controller.connection_spec",
+                c.controller_connection_spec.inspect)
+      dump_item("controller.unix_socket_mode",
+                "0%o" % c.controller_unix_socket_mode)
+      dump_item("controller.unix_socket_group",
+                c.controller_unix_socket_group.inspect)
+      dump_item("controller.remove_unix_socket_on_create",
+                c.remove_controller_unix_socket_on_create?)
+      dump_item("controller.remove_unix_socket_on_close",
+                c.remove_controller_unix_socket_on_close?)
+      @result << "\n"
+
+      c.applicable_conditions.each do |condition|
+        dump_applicable_condition(condition)
+        @result << "\n"
+      end
+
+      c.eggs.each do |egg|
+        dump_egg(egg)
+        @result << "\n"
+      end
+
+      @result.strip
+    end
+
+    private
+    def dump_location(key, indent="")
+      if key.nil?
+        location = nil
+      else
+        location = @locations[key]
+      end
+      if location
+        @result << "#{indent}# #{location['file']}:#{location['line']}\n"
+      else
+        @result << "#{indent}# default\n"
+      end
+    end
+
+    def dump_item(key, value, indent="", location_key=nil)
+      dump_location(location_key || key, indent)
+      @result << "#{indent}#{key} = #{value}\n"
+    end
+
+    def dump_applicable_condition_item(name, key, value)
+      dump_item("condition.#{key}",
+                value,
+                "  ",
+                "applicable_condition[#{name}].#{key}")
+    end
+
+    def dump_applicable_condition(condition)
+      name = condition.name
+      dump_location("applicable_condition[#{name}]")
+      @result << "define_applicable_condition(#{name.inspect}) do |condition|\n"
+      dump_applicable_condition_item(name, "description",
+                                     condition.description.inspect)
+      @result << "end\n"
+    end
+
+    def dump_egg_item(name, key, value)
+      dump_item("milter.#{key}",
+                value,
+                "  ",
+                "milter[#{name}].#{key}")
+    end
+
+    def dump_egg_applicable_conditions(egg)
+      conditions = egg.applicable_conditions
+      if conditions.empty?
+        dump_location(nil, "  ")
+        @result << "  milter.applicable_conditions = []\n"
+      else
+        egg_name = egg.name
+        @result << "  milter.applicable_conditions = [\n"
+        conditions.each do |condition|
+          name = condition.name
+          dump_location("milter[#{egg.name}].applicable_condition[#{name}]",
+                        "    ")
+          @result << "    #{name.inspect},\n"
+        end
+        @result << "  ]\n"
+      end
+    end
+
+    def dump_egg(egg)
+      name = egg.name
+      dump_location("milter[#{name}]")
+      @result << "define_milter(#{name.inspect}) do |milter|\n"
+      dump_egg_item(name, "connection_spec", egg.connection_spec.inspect)
+      dump_egg_item(name, "description", egg.description.inspect)
+      dump_egg_item(name, "enabled", egg.enabled?)
+      dump_egg_item(name, "fallback_status", egg.fallback_status.nick.inspect)
+      dump_egg_item(name, "evaluation_mode", egg.evaluation_mode?)
+      dump_egg_applicable_conditions(egg)
+      dump_egg_item(name, "command", egg.command.inspect)
+      dump_egg_item(name, "command_options", egg.command_options.inspect)
+      dump_egg_item(name, "user_name", egg.user_name.inspect)
+      dump_egg_item(name, "connection_timeout", egg.connection_timeout)
+      dump_egg_item(name, "writing_timeout", egg.writing_timeout)
+      dump_egg_item(name, "reading_timeout", egg.reading_timeout)
+      dump_egg_item(name, "end_of_message_timeout", egg.end_of_message_timeout)
+      @result << "end\n"
+    end
   end
 
   class ConfigurationLoader
@@ -164,20 +267,35 @@ module Milter::Manager
         Milter::Logger.error(error)
         fallback_value
       end
+
+      def update_location(configuration, key, reset, deep_level)
+        if reset
+          configuration.reset_location(key)
+        else
+          file, line, _ = caller[deep_level].split(/:/, 3)
+          configuration.set_location(key, file, line.to_i)
+        end
+      end
     end
 
     attr_reader :package, :security, :controller, :manager, :configuration
     def initialize(configuration)
+      @load_level = 0
       @configuration = configuration
       @package = PackageConfiguration.new(configuration)
       @security = SecurityConfiguration.new(configuration)
       @controller = ControllerConfiguration.new(configuration)
       @manager = ManagerConfiguration.new(configuration)
+      @policy_manager = Milter::Manager::PolicyManager.new(self)
     end
 
     def load_configuration(file)
       begin
-        instance_eval(File.read(file), file)
+        @load_level += 1
+        content = File.read(file)
+        Milter::Logger.debug("loading configuration file: '#{file}'")
+        instance_eval(content, file)
+        apply_policies if @load_level == 1
       rescue InvalidValue
         backtrace = $!.backtrace.collect do |info|
           if /\A#{Regexp.escape(file)}:/ =~ info
@@ -188,6 +306,8 @@ module Milter::Manager
         end.compact
         Milter::Logger.error("#{backtrace[0]}: #{$!.message}")
         Milter::Logger.error(backtrace[1..-1].join("\n"))
+      ensure
+        @load_level -= 1
       end
     end
 
@@ -196,10 +316,10 @@ module Milter::Manager
       case first_line
       when /\A\s*</m
         XMLConfigurationLoader.new(self).load(file)
-#       when /\A#\s*-\*-\s*yaml\s*-\*-/i
-#         YAMLConfigurationLoader.new(self).load(file)
-      else
-        load_configuration(file)
+      # when /\A#\s*-\*-\s*yaml\s*-\*-/i
+      #   YAMLConfigurationLoader.new(self).load(file)
+      # else
+      #   load_configuration(file)
       end
     end
 
@@ -237,12 +357,23 @@ module Milter::Manager
     end
 
     def define_milter(name, &block)
+      ConfigurationLoader.update_location(@configuration,
+                                          "milter[#{name}]",
+                                          false, 1)
       egg_configuration = EggConfiguration.new(name, self)
       yield(egg_configuration)
       egg_configuration.apply
     end
 
+    # more better name is needed?
+    def remove_milter(name)
+      @configuration.remove_egg(name)
+    end
+
     def define_applicable_condition(name)
+      ConfigurationLoader.update_location(@configuration,
+                                          "applicable_condition[#{name}]",
+                                          false, 1)
       condition_configuration = ApplicableConditionConfiguration.new(name, self)
       yield(condition_configuration)
       condition_configuration.apply
@@ -250,10 +381,16 @@ module Milter::Manager
 
     def defined_milters
       @configuration.eggs.collect do |egg|
-         egg.name
+        egg.name
       end
     end
 
+    def define_policy(name)
+      policy = Policy.new(name)
+      yield(policy)
+      @policy_manager << policy
+    end
+
     private
     def resolve_path(path)
       return [path] if Pathname(path).absolute?
@@ -271,6 +408,10 @@ module Milter::Manager
       resolved_paths
     end
 
+    def apply_policies
+      @policy_manager.apply
+    end
+
     class XMLConfigurationLoader
       def initialize(loader)
         @loader = loader
@@ -279,6 +420,7 @@ module Milter::Manager
       def load(file)
         listener = Listener.new(@loader)
         File.open(file) do |input|
+          Milter::Logger.debug("loading XML configuration file: '#{file}'")
           parser = REXML::Parsers::StreamParser.new(input, listener)
           begin
             parser.parse
@@ -286,7 +428,7 @@ module Milter::Manager
             source = parser.source
             info = "#{file}:#{source.line}:#{source.position}"
             info << ":#{source.current_line}: #{$!.message}"
-            Logger.error(info)
+            Milter::Logger.error(info)
           end
         end
       end
@@ -511,6 +653,7 @@ module Milter::Manager
                                  available_values,
                                  mode)
         end
+        update_location("privilege_mode", false)
         @configuration.privilege_mode = mode
       end
 
@@ -519,6 +662,7 @@ module Milter::Manager
       end
 
       def effective_user=(user_name)
+        update_location("effective_user", user_name.nil?)
         @configuration.effective_user = user_name
       end
 
@@ -527,8 +671,16 @@ module Milter::Manager
       end
 
       def effective_group=(group_name)
+        update_location("effective_group", group_name.nil?)
         @configuration.effective_group = group_name
       end
+
+      private
+      def update_location(key, reset, deep_level=2)
+        full_key = "security.#{key}"
+        ConfigurationLoader.update_location(@configuration, full_key, reset,
+                                            deep_level)
+      end
     end
 
     class ControllerConfiguration
@@ -538,6 +690,7 @@ module Milter::Manager
 
       def connection_spec=(spec)
         Milter::Connection.parse_spec(spec) unless spec.nil?
+        update_location("connection_spec", spec.nil?)
         @configuration.controller_connection_spec = spec
       end
 
@@ -546,6 +699,7 @@ module Milter::Manager
       end
 
       def unix_socket_mode=(mode)
+        update_location("unix_socket_mode", false)
         @configuration.controller_unix_socket_mode = mode
       end
 
@@ -554,6 +708,7 @@ module Milter::Manager
       end
 
       def unix_socket_group=(group)
+        update_location("unix_socket_group", group.nil?)
         @configuration.controller_unix_socket_group = group
       end
 
@@ -562,6 +717,7 @@ module Milter::Manager
       end
 
       def remove_unix_socket_on_create=(remove)
+        update_location("remove_unix_socket_on_create", false)
         @configuration.remove_controller_unix_socket_on_create = remove
       end
 
@@ -570,17 +726,27 @@ module Milter::Manager
       end
 
       def remove_unix_socket_on_close=(remove)
+        update_location("remove_unix_socket_on_close", false)
         @configuration.remove_controller_unix_socket_on_close = remove
       end
+
+      private
+      def update_location(key, reset, deep_level=2)
+        full_key = "controller.#{key}"
+        ConfigurationLoader.update_location(@configuration, full_key, reset,
+                                            deep_level)
+      end
     end
 
     class ManagerConfiguration
       def initialize(configuration)
         @configuration = configuration
+        @connection_checker_ids = {}
       end
 
       def connection_spec=(spec)
         Milter::Connection.parse_spec(spec) unless spec.nil?
+        update_location("connection_spec", spec.nil?)
         @configuration.manager_connection_spec = spec
       end
 
@@ -589,6 +755,7 @@ module Milter::Manager
       end
 
       def unix_socket_mode=(mode)
+        update_location("unix_socket_mode", false)
         @configuration.manager_unix_socket_mode = mode
       end
 
@@ -597,6 +764,7 @@ module Milter::Manager
       end
 
       def unix_socket_group=(group)
+        update_location("unix_socket_group", group.nil?)
         @configuration.manager_unix_socket_group = group
       end
 
@@ -605,6 +773,7 @@ module Milter::Manager
       end
 
       def remove_unix_socket_on_create=(remove)
+        update_location("remove_unix_socket_on_create", false)
         @configuration.remove_manager_unix_socket_on_create = remove
       end
 
@@ -613,10 +782,12 @@ module Milter::Manager
       end
 
       def remove_unix_socket_on_close=(remove)
+        update_location("remove_unix_socket_on_close", false)
         @configuration.remove_manager_unix_socket_on_close = remove
       end
 
       def daemon=(boolean)
+        update_location("daemon", false)
         @configuration.daemon = boolean
       end
 
@@ -625,6 +796,7 @@ module Milter::Manager
       end
 
       def pid_file=(pid_file)
+        update_location("pid_file", pid_file.nil?)
         @configuration.pid_file = pid_file
       end
 
@@ -633,6 +805,7 @@ module Milter::Manager
       end
 
       def maintenance_interval=(n_sessions)
+        update_location("maintenance_interval", n_sessions.nil?)
         n_sessions ||= 0
         @configuration.maintenance_interval = n_sessions
       end
@@ -642,6 +815,7 @@ module Milter::Manager
       end
 
       def suspend_time_on_unacceptable=(seconds)
+        update_location("suspend_time_on_unacceptable", seconds.nil?)
         seconds ||= Milter::Client::DEFAULT_SUSPEND_TIME_ON_UNACCEPTABLE
         @configuration.suspend_time_on_unacceptable = seconds
       end
@@ -651,6 +825,7 @@ module Milter::Manager
       end
 
       def max_connections=(n_connections)
+        update_location("max_connections", n_connections.nil?)
         n_connections ||= Milter::Client::DEFAULT_MAX_CONNECTIONS
         @configuration.max_connections = n_connections
       end
@@ -660,6 +835,7 @@ module Milter::Manager
       end
 
       def max_file_descriptors=(n_descriptors)
+        update_location("max_file_descriptors", n_descriptors.nil?)
         n_descriptors ||= 0
         @configuration.max_file_descriptors = n_descriptors
       end
@@ -669,12 +845,97 @@ module Milter::Manager
       end
 
       def custom_configuration_directory=(directory)
+        update_location("custom_configuration_directory", directory.nil?)
         @configuration.custom_configuration_directory = directory
       end
 
       def custom_configuration_directory
         @configuration.custom_configuration_directory
       end
+
+      def connection_check_interval=(interval)
+        update_location("connection_check_interval", interval.nil?)
+        interval ||= 0
+        @configuration.connection_check_interval = interval
+      end
+
+      def define_connection_checker(name, &block)
+        old_id = @connection_checker_ids[name]
+        @configuration.signal_handler_disconnect(old_id) if old_id
+        id = @configuration.signal_connect('connected') do |_config, leader|
+          leader.signal_connect('connection-check') do |_leader|
+            ConfigurationLoader.guard(true) do
+              block.call(ConnectionCheckContext.new(leader))
+            end
+          end
+        end
+        @connection_checker_ids[name] = id
+      end
+
+      def use_netstat_connection_checker(interval=5)
+        self.connection_check_interval = interval
+        checker = NetstatConnectionChecker.new(:database_lifetime => interval)
+        self.define_connection_checker("netstat") do |context|
+          checker.connected?(context)
+        end
+      end
+
+      def maintained(hook=Proc.new)
+        guarded_hook = Proc.new do |configuration|
+          ConfigurationLoader.guard do
+            hook.call
+          end
+        end
+        @configuration.maintained_hooks << guarded_hook
+      end
+
+      def report_memory_statistics
+        memory_reporter = MemoryReporter.new
+        maintained do
+          memory_reporter.report
+        end
+      end
+
+      private
+      def update_location(key, reset, deep_level=2)
+        full_key = "manager.#{key}"
+        ConfigurationLoader.update_location(@configuration, full_key, reset,
+                                            deep_level)
+      end
+
+      class MemoryReporter
+        def report
+          n_objects = count_alive_objects
+          message = "[maintain][memory] (#{memory_usage_in_kb}KB) "
+          message << "total:#{n_objects[:total]} "
+          message << "Proc:#{n_objects[:proc]} "
+          message << "GLib::Object:#{n_objects[:glib_object]}"
+          Milter::Logger.statistics(message)
+        end
+
+        private
+        def count_alive_objects
+          n_procs = 0
+          n_glib_objects = 0
+          n_objects = ObjectSpace.each_object do |object|
+            case object
+            when Proc
+              n_procs += 1
+            when GLib::Object
+              n_glib_objects += 1
+            end
+          end
+          {
+            :total => n_objects,
+            :proc => n_procs,
+            :glib_object => n_glib_objects,
+          }
+        end
+
+        def memory_usage_in_kb
+          `ps -o rss -p #{Process.pid}`.split(/\n/).last.to_i
+        end
+      end
     end
 
     class EggConfiguration
@@ -695,6 +956,7 @@ module Milter::Manager
                                  conditions.collect {|cond| cond.name},
                                  name)
         end
+        update_location("applicable_condition[#{name}]", false)
         @egg.add_applicable_condition(condition)
       end
 
@@ -716,6 +978,9 @@ module Milter::Manager
           end
           condition
         end
+        @egg.applicable_conditions.each do |condition|
+          update_location("applicable_condition[#{condition.name}]", true)
+        end
         @egg.clear_applicable_conditions
         conditions.each do |condition|
           @egg.add_applicable_condition(condition)
@@ -728,6 +993,7 @@ module Milter::Manager
             Shellwords.escape(option)
           end.join(' ')
         end
+        update_location("command_options", options.empty?)
         @egg.command_options = options
       end
 
@@ -754,17 +1020,29 @@ module Milter::Manager
                                  available_values.keys,
                                  status)
         end
+        update_location("fallback_status", false)
         @egg.fallback_status = value
       end
 
       def method_missing(name, *args, &block)
-        @egg.send(name, *args, &block)
+        result = @egg.send(name, *args, &block)
+        if /=\z/ =~ name.to_s
+          update_location($PREMATCH, args[0].nil?)
+        end
+        result
       end
 
       def apply
         @loader.configuration.remove_egg(@egg.name)
         @loader.configuration.add_egg(@egg)
       end
+
+      private
+      def update_location(name, reset, deep_level=2)
+        full_key = "milter[#{@egg.name}].#{name}"
+        ConfigurationLoader.update_location(@loader.configuration,
+                                            full_key, reset, deep_level)
+      end
     end
 
     class ApplicableConditionConfiguration
@@ -837,7 +1115,11 @@ module Milter::Manager
       end
 
       def method_missing(name, *args, &block)
-        @condition.send(name, *args, &block)
+        result = @condition.send(name, *args, &block)
+        if /=\z/ =~ name.to_s
+          update_location($PREMATCH, args[0].nil?, 2)
+        end
+        result
       end
 
       def apply
@@ -850,13 +1132,13 @@ module Milter::Manager
       def setup_stoppers
         return unless have_stopper?
 
-        @condition.signal_connect("attach-to") do |_, child, children|
+        @condition.signal_connect("attach-to") do |_, child, children, context|
           unless @connect_stoppers.empty?
             child.signal_connect("stop-on-connect") do |_child, host, address|
-              context = ChildContext.new(child, children)
+              child_context = ChildContext.new(child, children, context)
               @connect_stoppers.any? do |stopper|
                 ConfigurationLoader.guard(false) do
-                  stopper.call(context, host, address)
+                  stopper.call(child_context, host, address)
                 end
               end
             end
@@ -864,10 +1146,10 @@ module Milter::Manager
 
           unless @helo_stoppers.empty?
             child.signal_connect("stop-on-helo") do |_child, fqdn|
-              context = ChildContext.new(child, children)
+              child_context = ChildContext.new(child, children, context)
               @helo_stoppers.any? do |stopper|
                 ConfigurationLoader.guard(false) do
-                  stopper.call(context, fqdn)
+                  stopper.call(child_context, fqdn)
                 end
               end
             end
@@ -875,10 +1157,10 @@ module Milter::Manager
 
           unless @envelope_from_stoppers.empty?
             child.signal_connect("stop-on-envelope-from") do |_child, from|
-              context = ChildContext.new(child, children)
+              child_context = ChildContext.new(child, children, context)
               @envelope_from_stoppers.any? do |stopper|
                 ConfigurationLoader.guard(false) do
-                  stopper.call(context, from)
+                  stopper.call(child_context, from)
                 end
               end
             end
@@ -886,10 +1168,10 @@ module Milter::Manager
 
           unless @envelope_recipient_stoppers.empty?
             child.signal_connect("stop-on-envelope-recipient") do |_child, recipient|
-              context = ChildContext.new(child, children)
+              child_context = ChildContext.new(child, children, context)
               @envelope_recipient_stoppers.any? do |stopper|
                 ConfigurationLoader.guard(false) do
-                  stopper.call(context, recipient)
+                  stopper.call(child_context, recipient)
                 end
               end
             end
@@ -897,10 +1179,10 @@ module Milter::Manager
 
           unless @data_stoppers.empty?
             child.signal_connect("stop-on-data") do |_child|
-              context = ChildContext.new(child, children)
+              child_context = ChildContext.new(child, children, context)
               @data_stoppers.any? do |stopper|
                 ConfigurationLoader.guard(false) do
-                  stopper.call(context)
+                  stopper.call(child_context)
                 end
               end
             end
@@ -908,10 +1190,10 @@ module Milter::Manager
 
           unless @header_stoppers.empty?
             child.signal_connect("stop-on-header") do |_child, name, value|
-              context = ChildContext.new(child, children)
+              child_context = ChildContext.new(child, children, context)
               @header_stoppers.any? do |stopper|
                 ConfigurationLoader.guard(false) do
-                  stopper.call(context, name, value)
+                  stopper.call(child_context, name, value)
                 end
               end
             end
@@ -919,10 +1201,10 @@ module Milter::Manager
 
           unless @end_of_header_stoppers.empty?
             child.signal_connect("stop-on-end-of-header") do |_child|
-              context = ChildContext.new(child, children)
+              child_context = ChildContext.new(child, children, context)
               @end_of_header_stoppers.any? do |stopper|
                 ConfigurationLoader.guard(false) do
-                  stopper.call(context)
+                  stopper.call(child_context)
                 end
               end
             end
@@ -930,10 +1212,10 @@ module Milter::Manager
 
           unless @body_stoppers.empty?
             child.signal_connect("stop-on-body") do |_child, chunk|
-              context = ChildContext.new(child, children)
+              child_context = ChildContext.new(child, children, context)
               @body_stoppers.any? do |stopper|
                 ConfigurationLoader.guard(false) do
-                  stopper.call(context, name, chunk)
+                  stopper.call(child_context, name, chunk)
                 end
               end
             end
@@ -941,16 +1223,22 @@ module Milter::Manager
 
           unless @end_of_message_stoppers.empty?
             child.signal_connect("stop-on-end-of-message") do |_child, chunk|
-              context = ChildContext.new(child, children)
+              child_context = ChildContext.new(child, children, context)
               @end_of_message_stoppers.any? do |stopper|
                 ConfigurationLoader.guard(false) do
-                  stopper.call(context)
+                  stopper.call(child_context)
                 end
               end
             end
           end
         end
       end
+
+      def update_location(name, reset, deep_level=2)
+        full_key = "applicable_condition[#{@condition.name}].#{name}"
+        ConfigurationLoader.update_location(@loader.configuration,
+                                            full_key, reset, deep_level)
+      end
     end
   end
 end
