@@ -15,9 +15,10 @@
 
 module Milter::Manager
   class ChildContext
-    def initialize(child, children)
+    def initialize(child, children, client_context)
       @child = child
       @children = children
+      @client_context = client_context
     end
 
     def name
@@ -74,11 +75,15 @@ module Milter::Manager
       end
     end
 
+    def n_processing_sessions
+      @client_context.n_processing_sessions
+    end
+
     private
     def create_child_contexts
       contexts = {}
       @children.children.each do |child|
-        contexts[child.name] = self.class.new(child, @children)
+        contexts[child.name] = self.class.new(child, @children, @client_context)
       end
       contexts
     end
