@@ -13,6 +13,8 @@
 # You should have received a copy of the GNU Lesser General Public License
 # along with this library.  If not, see <http://www.gnu.org/licenses/>.
 
+require 'ipaddr'
+
 module Milter
   class Logger
     @@domain = "milter-manager"
@@ -49,19 +51,19 @@ module Milter
         default.log(:info, message, 1)
       end
 
-      def statistics(message, n_call_depth=nil)
+      def statistics(message)
         default.log(:statistics, message, 1)
       end
     end
 
     def log(level, message, n_call_depth=nil)
       unless level.is_a?(Milter::LogLevelFlags)
-        level = Milter::LogLevelFlags.from_string(level.to_s)
+        level = Milter::LogLevelFlags.const_get(level.to_s.upcase)
       end
       n_call_depth ||= 0
       file, line, info = caller[n_call_depth].split(/:(\d+):/, 3)
       ensure_message(message).each_line do |one_line_message|
-        log_full(self.class.domain, level, file, line.to_i, info,
+        log_full(self.class.domain, level, file, line.to_i, info.to_s,
                  one_line_message.chomp)
       end
     end
@@ -91,6 +93,14 @@ module Milter
         return true if bit1 == 192 and bit2 == 168
         false
       end
+
+      def unknown?
+        false
+      end
+
+      def to_ip_address
+        @ip_address ||= IPAddr.new(address)
+      end
     end
 
     class IPv6
@@ -105,18 +115,42 @@ module Milter
         return true if bits[0] == 0xfe80
         false
       end
+
+      def unknown?
+        false
+      end
+
+      def to_ip_address
+        @ip_address ||= IPAddr.new(address)
+      end
     end
 
     class Unix
       def local?
         true
       end
-    end
+
+      def unknown?
+        false
+      end
+
+      def to_ip_address
+        nil
+      end
+   end
 
     class Unknown
       def local?
         false
       end
+
+      def unknown?
+        true
+      end
+
+      def to_ip_address
+        nil
+      end
     end
   end
 
