@@ -1,6 +1,6 @@
 /* -*- c-file-style: "ruby" -*- */
 /*
- *  Copyright (C) 2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -17,6 +17,7 @@
  *
  */
 
+#include "../toolkit/rb-milter-core-private.h"
 #include "rb-milter-manager-private.h"
 
 #define SELF(self) (MILTER_MANAGER_CHILDREN(RVAL2GOBJ(self)))
@@ -74,13 +75,22 @@ get_quitted_children (VALUE self)
     return GLIST2ARY(milter_manager_children_get_quitted_children(SELF(self)));
 }
 
-static void
-mark (gpointer data)
+static VALUE
+get_smtp_client_address (VALUE self)
 {
-    MilterManagerChildren *children = data;
+    VALUE rb_address;
+    struct sockaddr *address;
+    socklen_t address_length;
 
-    g_list_foreach((GList *)milter_manager_children_get_children(children),
-		   (GFunc)rbgobj_gc_mark_instance, NULL);
+    if (milter_manager_children_get_smtp_client_address(SELF(self),
+							&address,
+							&address_length)) {
+	rb_address = ADDRESS2RVAL(address, address_length);
+	g_free(address);
+	return rb_address;
+    } else {
+	return Qnil;
+    }
 }
 
 void
@@ -89,11 +99,7 @@ Init_milter_manager_children (void)
     VALUE rb_cMilterManagerChildren;
 
     rb_cMilterManagerChildren =
-	G_DEF_CLASS_WITH_GC_FUNC(MILTER_TYPE_MANAGER_CHILDREN,
-				 "Children",
-				 rb_mMilterManager,
-				 mark,
-				 NULL);
+	G_DEF_CLASS(MILTER_TYPE_MANAGER_CHILDREN, "Children", rb_mMilterManager);
 
     rb_define_method(rb_cMilterManagerChildren, "initialize",
                      initialize, 1);
@@ -104,4 +110,6 @@ Init_milter_manager_children (void)
     rb_define_method(rb_cMilterManagerChildren, "children", get_children, 0);
     rb_define_method(rb_cMilterManagerChildren, "quitted_children",
 		     get_quitted_children, 0);
+    rb_define_method(rb_cMilterManagerChildren, "smtp_client_address",
+		     get_smtp_client_address, 0);
 }
