@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+# Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
 #
 # This library is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -119,7 +119,7 @@ class TestConfigurationLoader < Test::Unit::TestCase
   end
 
   def test_manager_maintenance_interval
-    assert_equal(100, @configuration.maintenance_interval)
+    assert_equal(10, @configuration.maintenance_interval)
 
     @loader.manager.maintenance_interval = nil
     assert_equal(0, @configuration.maintenance_interval)
@@ -127,8 +127,8 @@ class TestConfigurationLoader < Test::Unit::TestCase
     @loader.manager.maintenance_interval = 0
     assert_equal(0, @configuration.maintenance_interval)
 
-    @loader.manager.maintenance_interval = 10
-    assert_equal(10, @configuration.maintenance_interval)
+    @loader.manager.maintenance_interval = 100
+    assert_equal(100, @configuration.maintenance_interval)
   end
 
   def test_manager_suspend_time_on_unacceptable
@@ -292,6 +292,14 @@ EOX
                  end)
   end
 
+  def test_remove_milter
+    assert_equal([], @loader.defined_milters)
+    @loader.define_milter("milter1") {}
+    assert_equal(["milter1"], @loader.defined_milters)
+    @loader.remove_milter("milter1")
+    assert_equal([], @loader.defined_milters)
+  end
+
   def test_defined_milters
     @loader.define_milter("milter1") do |milter|
       milter.description = "description1"
