@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+# Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
 #
 # This library is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -34,7 +34,7 @@ class TestConfiguration < Test::Unit::TestCase
   end
 
   def test_maintenance_interval
-    assert_equal(100, @configuration.maintenance_interval)
+    assert_equal(10, @configuration.maintenance_interval)
     @configuration.maintenance_interval = 0
     assert_equal(0, @configuration.maintenance_interval)
   end
@@ -57,25 +57,40 @@ class TestConfiguration < Test::Unit::TestCase
     assert_equal(10, @configuration.max_file_descriptors)
   end
 
+  def test_max_connection_check_interval
+    assert_equal(0, @configuration.connection_check_interval)
+    @configuration.connection_check_interval = 10
+    assert_equal(10, @configuration.connection_check_interval)
+  end
+
   def test_package
     @configuration.package_platform = "pkgsrc"
     assert_equal("pkgsrc", @configuration.package_platform)
     @configuration.package_options = "prefix=/etc"
     assert_equal("prefix=/etc", @configuration.package_options)
+    assert_equal({"prefix" => "/etc"}, @configuration.parsed_package_options)
+  end
+
+  def test_package_options_with_spaces
+    @configuration.package_options = "prefix = /etc , DESTDIR = /tmp/local"
+    assert_equal("prefix = /etc , DESTDIR = /tmp/local",
+                 @configuration.package_options)
+    assert_equal({"prefix" => "/etc", "DESTDIR" => "/tmp/local"},
+                 @configuration.parsed_package_options)
   end
 
   def test_add_egg
-    assert_equal(0, @configuration.create_children.length)
+    assert_equal(0, create_children.length)
 
     name = "child-milter"
     egg = Milter::Manager::Egg.new(name)
     egg.connection_spec = "unix:/tmp/socket"
     @configuration.add_egg(egg)
-    assert_equal(1, @configuration.create_children.length)
+    assert_equal(1, create_children.length)
 
     @configuration.clear_eggs
     assert_equal([], @configuration.eggs)
-    assert_equal(0, @configuration.create_children.length)
+    assert_equal(0, create_children.length)
   end
 
   def test_find_egg
@@ -198,30 +213,54 @@ class TestConfiguration < Test::Unit::TestCase
 
   def test_dump
     assert_equal(<<-EOD.strip,
+# default
 package.platform = #{@configuration.package_platform.inspect}
+# default
 package.options = #{@configuration.package_options.inspect}
 
+# default
 security.privilege_mode = false
+# default
 security.effective_user = nil
+# default
 security.effective_group = nil
 
+# default
 manager.connection_spec = #{@configuration.manager_connection_spec.inspect}
+# default
 manager.unix_socket_mode = 0660
+# default
 manager.unix_socket_group = nil
+# default
 manager.remove_unix_socket_on_create = true
+# default
 manager.remove_unix_socket_on_close = true
+# default
 manager.daemon = false
+# default
 manager.pid_file = nil
-manager.maintenance_interval = 100
+# default
+manager.maintenance_interval = 10
+# default
 manager.suspend_time_on_unacceptable = 5
+# default
 manager.max_connections = 0
+# default
 manager.max_file_descriptors = 0
+# default
 manager.custom_configuration_directory = nil
+# default
+manager.connection_check_interval = 0
 
+# default
 controller.connection_spec = nil
+# default
 controller.unix_socket_mode = 0660
+# default
 controller.unix_socket_group = nil
+# default
 controller.remove_unix_socket_on_create = true
+# default
 controller.remove_unix_socket_on_close = true
 EOD
                  @configuration.dump)
@@ -229,89 +268,200 @@ EOD
 
   def test_dump_full
     loader = Milter::Manager::ConfigurationLoader.new(@configuration)
-    loader.security.effective_user = "nobody"
-    loader.security.effective_group = "nogroup"
+    loader.security.effective_user = "nobody"; effective_user_line = __LINE__
+    loader.security.effective_group = "nogroup"; effective_group_line = __LINE__
 
+    manager_unix_socket_group = __LINE__ + 1
+    loader.manager.unix_socket_group = "nogroup"
+    manager_pid_file = __LINE__ + 1
+    loader.manager.pid_file = "/tmp/milter-manager.pid"
+    manager_custom_configuration_directory = __LINE__ + 1
+    loader.manager.custom_configuration_directory = "/tmp/milter-manager/"
+
+    controller_connection_spec = __LINE__ + 1
+    loader.controller.connection_spec = "inet:10025"
+    controller_unix_socket_group = __LINE__ + 1
+    loader.controller.unix_socket_group = "nogroup"
+
+    s25r_lines = {}
+    s25r_lines[:define] = __LINE__ + 1
     loader.define_applicable_condition("S25R") do |condition|
+      s25r_lines[:description] = __LINE__ + 1
       condition.description = "Selective SMTP Rejection"
     end
 
+    milter1_lines = {}
+    milter1_lines[:define] = __LINE__ + 1
     loader.define_milter("milter1") do |milter|
+      milter1_lines[:description] = __LINE__ + 1
       milter.description = "The first milter"
+      milter1_lines[:connection_spec] = __LINE__ + 1
       milter.connection_spec = "unix:/tmp/xxx"
+      milter1_lines[:applicable_condition_s25r] = __LINE__ + 1
       milter.add_applicable_condition("S25R")
     end
 
+    milter2_lines = {}
+    milter2_lines[:define] = __LINE__ + 1
     loader.define_milter("milter2") do |milter|
+      milter2_lines[:description] = __LINE__ + 1
       milter.description = "The second milter"
+      milter2_lines[:connection_spec] = __LINE__ + 1
       milter.connection_spec = "inet:2929"
+      milter2_lines[:enabled] = __LINE__ + 1
       milter.enabled = false
+      milter2_lines[:evaluation_mode] = __LINE__ + 1
       milter.evaluation_mode = true
     end
 
     assert_equal(<<-EOD.strip,
+# default
 package.platform = #{@configuration.package_platform.inspect}
+# default
 package.options = #{@configuration.package_options.inspect}
 
+# default
 security.privilege_mode = false
+# #{__FILE__}:#{effective_user_line}
 security.effective_user = "nobody"
+# #{__FILE__}:#{effective_group_line}
 security.effective_group = "nogroup"
 
+# default
 manager.connection_spec = #{@configuration.manager_connection_spec.inspect}
+# default
 manager.unix_socket_mode = 0660
-manager.unix_socket_group = nil
+# #{__FILE__}:#{manager_unix_socket_group}
+manager.unix_socket_group = "nogroup"
+# default
 manager.remove_unix_socket_on_create = true
+# default
 manager.remove_unix_socket_on_close = true
+# default
 manager.daemon = false
-manager.pid_file = nil
-manager.maintenance_interval = 100
+# #{__FILE__}:#{manager_pid_file}
+manager.pid_file = "/tmp/milter-manager.pid"
+# default
+manager.maintenance_interval = 10
+# default
 manager.suspend_time_on_unacceptable = 5
+# default
 manager.max_connections = 0
+# default
 manager.max_file_descriptors = 0
-manager.custom_configuration_directory = nil
+# #{__FILE__}:#{manager_custom_configuration_directory}
+manager.custom_configuration_directory = "/tmp/milter-manager/"
+# default
+manager.connection_check_interval = 0
 
-controller.connection_spec = nil
+# #{__FILE__}:#{controller_connection_spec}
+controller.connection_spec = "inet:10025"
+# default
 controller.unix_socket_mode = 0660
-controller.unix_socket_group = nil
+# #{__FILE__}:#{controller_unix_socket_group}
+controller.unix_socket_group = "nogroup"
+# default
 controller.remove_unix_socket_on_create = true
+# default
 controller.remove_unix_socket_on_close = true
 
+# #{__FILE__}:#{s25r_lines[:define]}
 define_applicable_condition("S25R") do |condition|
+  # #{__FILE__}:#{s25r_lines[:description]}
   condition.description = "Selective SMTP Rejection"
 end
 
+# #{__FILE__}:#{milter1_lines[:define]}
 define_milter("milter1") do |milter|
+  # #{__FILE__}:#{milter1_lines[:connection_spec]}
   milter.connection_spec = "unix:/tmp/xxx"
+  # #{__FILE__}:#{milter1_lines[:description]}
   milter.description = "The first milter"
+  # default
   milter.enabled = true
+  # default
   milter.fallback_status = "accept"
+  # default
   milter.evaluation_mode = false
-  milter.applicable_conditions = ["S25R"]
+  milter.applicable_conditions = [
+    # #{__FILE__}:#{milter1_lines[:applicable_condition_s25r]}
+    "S25R",
+  ]
+  # default
   milter.command = nil
+  # default
   milter.command_options = nil
+  # default
   milter.user_name = nil
+  # default
   milter.connection_timeout = 300.0
+  # default
   milter.writing_timeout = 10.0
+  # default
   milter.reading_timeout = 10.0
+  # default
   milter.end_of_message_timeout = 300.0
 end
 
+# #{__FILE__}:#{milter2_lines[:define]}
 define_milter("milter2") do |milter|
+  # #{__FILE__}:#{milter2_lines[:connection_spec]}
   milter.connection_spec = "inet:2929"
+  # #{__FILE__}:#{milter2_lines[:description]}
   milter.description = "The second milter"
+  # #{__FILE__}:#{milter2_lines[:enabled]}
   milter.enabled = false
+  # default
   milter.fallback_status = "accept"
+  # #{__FILE__}:#{milter2_lines[:evaluation_mode]}
   milter.evaluation_mode = true
+  # default
   milter.applicable_conditions = []
+  # default
   milter.command = nil
+  # default
   milter.command_options = nil
+  # default
   milter.user_name = nil
+  # default
   milter.connection_timeout = 300.0
+  # default
   milter.writing_timeout = 10.0
+  # default
   milter.reading_timeout = 10.0
+  # default
   milter.end_of_message_timeout = 300.0
 end
 EOD
                  @configuration.dump)
   end
+
+  def test_locations
+    assert_equal({}, @configuration.locations)
+    @configuration.set_location("security.privilege_mode",
+                                "milter-manager.conf", 29)
+    assert_equal({
+                   "file" => "milter-manager.conf",
+                   "line" => 29
+                 },
+                 @configuration.get_location("security.privilege_mode"))
+    assert_equal({
+                   "security.privilege_mode" => {
+                     "file" => "milter-manager.conf",
+                     "line" => 29
+                   }
+                 },
+                 @configuration.locations)
+
+    @configuration.reset_location("security.privilege_mode")
+    assert_equal({}, @configuration.locations)
+  end
+
+  private
+  def create_children
+    children = Milter::Manager::Children.new(@configuration)
+    client_context = Milter::ClientContext.new
+    @configuration.setup_children(children, client_context)
+    children
+  end
 end
