@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2009  Kouhei Sutou <kou@clear-code.com>
+# Copyright (C) 2009-2010  Kouhei Sutou <kou@clear-code.com>
 #
 # This library is free software: you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -133,15 +133,20 @@ EOC
   end
 
   private
-  def redhat_upstart_detector(name)
+  def redhat_upstart_detector(name, options={})
     detector = Milter::Manager::RedHatUpstartDetector.new(@configuration, name)
 
     _etc_dir = @etc_dir
+    _candidate_service_commands = options[:candidate_service_commands] || []
     singleton_object = class << detector; self; end
     singleton_object.send(:define_method, :etc_dir) do
       _etc_dir.to_s
     end
 
+    singleton_object.send(:define_method, :candidate_service_commands) do
+      _candidate_service_commands
+    end
+
     detector
   end
 
