@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -21,6 +21,7 @@
 #define __MILTER_CLIENT_H__
 
 #include <milter/core.h>
+#include <milter/client/milter-client-objects.h>
 #include <milter/client/milter-client-context.h>
 #include <milter/client/milter-client-enum-types.h>
 
@@ -207,7 +208,6 @@ typedef enum
     MILTER_CLIENT_ERROR_DAEMONIZE
 } MilterClientError;
 
-typedef struct _MilterClient         MilterClient;
 typedef struct _MilterClientClass    MilterClientClass;
 
 /**
@@ -249,6 +249,12 @@ struct _MilterClientClass
     const gchar *(*get_effective_group)   (MilterClient *client);
     void   (*set_effective_group)         (MilterClient *client,
                                            const gchar  *effective_group);
+    guint  (*get_maintenance_interval)    (MilterClient *client);
+    void   (*set_maintenance_interval)    (MilterClient *client,
+                                           guint         maintenance_interval);
+    void   (*maintain)                    (MilterClient *client);
+    void   (*sessions_finished)           (MilterClient *client,
+                                           guint         n_finished_sessions);
 };
 
 GQuark               milter_client_error_quark       (void);
@@ -593,6 +599,35 @@ void                 milter_client_set_effective_group
                                                      (MilterClient  *client,
                                                       const gchar   *effective_group);
 
+/**
+ * milter_client_get_maintenance_interval:
+ * @client: a %MilterClient.
+ *
+ * Gets the maintenance interval. #MilterClient::maintain
+ * signal is emitted each 'maintenance interval' sessions
+ * finished.
+ *
+ * 0 means 'maintain signal isn't needed'.
+ *
+ * Returns: the maintenance interval.
+ */
+guint                milter_client_get_maintenance_interval
+                                                     (MilterClient  *client);
+
+/**
+ * milter_client_set_maintenance_interval:
+ * @client: a %MilterClient.
+ * @maintenance_interval: the maintenance interval.
+ *
+ * Sets the maintenance interval. 0 means 'maintain
+ * signal isn't needed'. See
+ * milter_client_get_maintenance_interval() for more
+ * details.
+ */
+void                 milter_client_set_maintenance_interval
+                                                     (MilterClient  *client,
+                                                      guint          n_sessions);
+
 /**
  * milter_client_main:
  * @client: a %MilterClient.
@@ -612,6 +647,21 @@ gboolean             milter_client_main              (MilterClient  *client);
  */
 void                 milter_client_shutdown          (MilterClient  *client);
 
+/**
+ * milter_client_processing_context_foreach:
+ * @client: a %MilterClient.
+ * @func: the function to call with each processing %MilterClientContext.
+ * @user_data: user data to pass to the function.
+ *
+ * Calls a function for each processing
+ * %MilterClientContext.
+ */
+void                 milter_client_processing_context_foreach
+                                                     (MilterClient  *client,
+                                                      GFunc          func,
+                                                      gpointer       user_data);
+
+
 /**
  * milter_client_drop_privilege:
  * @client: a %MilterClient.
@@ -636,6 +686,17 @@ gboolean             milter_client_drop_privilege    (MilterClient  *client,
 gboolean             milter_client_daemonize         (MilterClient  *client,
                                                       GError       **error);
 
+/**
+ * milter_client_get_n_processing_sessions:
+ * @client: a %MilterClient.
+ *
+ * Returns number of the current processing sessions.
+ *
+ * Returns: number of the current processing sessions.
+ */
+guint                milter_client_get_n_processing_sessions
+                                                     (MilterClient  *client);
+
 G_END_DECLS
 
 #endif /* __MILTER_CLIENT_H__ */
