@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -34,19 +34,32 @@
 
 #include <milter/core/milter-marshalers.h>
 #include "../client.h"
+#include "milter-client-private.h"
 
+static gboolean initialized = FALSE;
 static guint milter_client_log_handler_id = 0;
 
 void
 milter_client_init (void)
 {
+    if (initialized)
+        return;
+
+    initialized = TRUE;
+
     milter_client_log_handler_id = MILTER_GLIB_LOG_DELEGATE("milter-client");
 }
 
 void
 milter_client_quit (void)
 {
+    if (!initialized)
+        return;
+
     g_log_remove_handler("milter-client", milter_client_log_handler_id);
+
+    milter_client_log_handler_id = 0;
+    initialized = FALSE;
 }
 
 
@@ -54,6 +67,8 @@ enum
 {
     CONNECTION_ESTABLISHED,
     LISTEN_STARTED,
+    MAINTAIN,
+    SESSIONS_FINISHED,
     LAST_SIGNAL
 };
 
@@ -73,7 +88,9 @@ struct _MilterClientPrivate
     guint server_watch_id;
     gchar *connection_spec;
     GList *processing_data;
-    guint n_connections;
+    guint n_processing_sessions;
+    guint n_processed_sessions;
+    guint maintenance_interval;
     guint timeout;
     GIOChannel *listen_channel;
     gint listen_backlog;
@@ -85,10 +102,15 @@ struct _MilterClientPrivate
     gboolean remove_unix_socket_on_create;
     guint suspend_time_on_unacceptable;
     guint max_connections;
+    gboolean multi_workers_mode;
+    GThreadPool *worker_threads;
     struct sockaddr *address;
     socklen_t address_size;
     gchar *effective_user;
     gchar *effective_group;
+
+    guint finisher_id;
+    GList *finished_data;
 };
 
 typedef struct _MilterClientProcessData
@@ -96,8 +118,11 @@ typedef struct _MilterClientProcessData
     MilterClientPrivate *priv;
     MilterClient *client;
     MilterClientContext *context;
+    gulong finished_handler_id;
 } MilterClientProcessData;
 
+typedef gboolean (*AcceptConnectionFunction) (MilterClient *client, gint fd);
+
 #define _milter_client_get_type milter_client_get_type
 MILTER_DEFINE_ERROR_EMITTABLE_TYPE(MilterClient, _milter_client, G_TYPE_OBJECT)
 #undef _milter_client_get_type
@@ -118,8 +143,6 @@ static void   listen_started
                            (MilterClient    *client,
                             struct sockaddr *address,
                             socklen_t        address_size);
-static void   cb_finished  (MilterClientContext *context,
-                            gpointer _data);
 
 static void
 _milter_client_class_init (MilterClientClass *klass)
@@ -155,6 +178,24 @@ _milter_client_class_init (MilterClientClass *klass)
                      _milter_marshal_VOID__POINTER_UINT,
                      G_TYPE_NONE, 2, G_TYPE_POINTER, G_TYPE_UINT);
 
+    signals[MAINTAIN] =
+        g_signal_new("maintain",
+                     MILTER_TYPE_CLIENT,
+                     G_SIGNAL_RUN_LAST,
+                     G_STRUCT_OFFSET(MilterClientClass, maintain),
+                     NULL, NULL,
+                     g_cclosure_marshal_VOID__VOID,
+                     G_TYPE_NONE, 0);
+
+    signals[SESSIONS_FINISHED] =
+        g_signal_new("sessions-finished",
+                     MILTER_TYPE_CLIENT,
+                     G_SIGNAL_RUN_LAST,
+                     G_STRUCT_OFFSET(MilterClientClass, sessions_finished),
+                     NULL, NULL,
+                     g_cclosure_marshal_VOID__UINT,
+                     G_TYPE_NONE, 1, G_TYPE_UINT);
+
     g_type_class_add_private(gobject_class, sizeof(MilterClientPrivate));
 }
 
@@ -172,12 +213,14 @@ _milter_client_init (MilterClient *client)
     priv->accept_loop = g_main_loop_new(main_context, FALSE);
     g_main_context_unref(main_context);
 
-    priv->main_loop = g_main_loop_new(NULL, FALSE);
+    priv->main_loop = NULL;
 
     priv->server_watch_id = 0;
     priv->connection_spec = NULL;
     priv->processing_data = NULL;
-    priv->n_connections = 0;
+    priv->n_processing_sessions = 0;
+    priv->n_processed_sessions = 0;
+    priv->maintenance_interval = 0;
     priv->timeout = 7210;
     priv->listen_channel = NULL;
     priv->listen_backlog = -1;
@@ -190,17 +233,31 @@ _milter_client_init (MilterClient *client)
     priv->suspend_time_on_unacceptable =
         MILTER_CLIENT_DEFAULT_SUSPEND_TIME_ON_UNACCEPTABLE;
     priv->max_connections = MILTER_CLIENT_DEFAULT_MAX_CONNECTIONS;
+    priv->multi_workers_mode = FALSE;
+    priv->worker_threads = NULL;
     priv->address = NULL;
     priv->address_size = 0;
     priv->effective_user = NULL;
     priv->effective_group = NULL;
+
+    priv->finisher_id = 0;
+    priv->finished_data = NULL;
+}
+
+static void
+dispose_process_data_finished_handler (MilterClientProcessData *data)
+{
+    if (data->finished_handler_id > 0) {
+        g_signal_handler_disconnect(data->context,
+                                    data->finished_handler_id);
+        data->finished_handler_id = 0;
+    }
 }
 
 static void
 process_data_free (MilterClientProcessData *data)
 {
-    g_signal_handlers_disconnect_by_func(data->context,
-                                         G_CALLBACK(cb_finished), data);
+    dispose_process_data_finished_handler(data);
     g_object_unref(data->context);
     g_free(data);
 }
@@ -215,6 +272,132 @@ dispose_address (MilterClientPrivate *priv)
     }
 }
 
+static void
+dispose_finisher (MilterClientPrivate *priv)
+{
+    if (priv->finisher_id > 0) {
+        g_source_remove(priv->finisher_id);
+        priv->finisher_id = 0;
+    }
+}
+
+static void
+finish_processing (MilterClientProcessData *data)
+{
+    guint n_processing_sessions;
+    GString *rest_process;
+    guint tag = 0;
+
+    if (milter_need_debug_log()) {
+        tag = milter_agent_get_tag(MILTER_AGENT(data->context));
+        milter_debug("[%u] [client][finish]", tag);
+    }
+
+    data->priv->processing_data =
+        g_list_remove(data->priv->processing_data, data);
+    data->priv->n_processing_sessions--;
+    data->priv->n_processed_sessions++;
+
+    if (data->priv->quitting && data->priv->main_loop) {
+        n_processing_sessions = data->priv->n_processing_sessions;
+        g_mutex_lock(data->priv->quit_mutex);
+        if (data->priv->quitting && n_processing_sessions == 0) {
+            milter_debug("[%u] [client][loop][quit]", tag);
+            g_main_loop_quit(data->priv->main_loop);
+        }
+        g_mutex_unlock(data->priv->quit_mutex);
+    }
+
+    if (milter_need_debug_log()) {
+        GList *processing_data, *process_data;
+
+        processing_data = g_list_copy(data->priv->processing_data);
+        rest_process = g_string_new("[");
+        for (process_data = processing_data;
+             process_data;
+             process_data = g_list_next(process_data)) {
+            MilterClientProcessData *_process_data = process_data->data;
+            g_string_append_printf(
+                rest_process, "<%u>, ",
+                milter_agent_get_tag(MILTER_AGENT(_process_data->context)));
+        }
+        if (processing_data)
+            g_string_truncate(rest_process, rest_process->len - 2);
+        g_string_append(rest_process, "]");
+        g_list_free(processing_data);
+        milter_debug("[%u] [client][rest] %s", tag, rest_process->str);
+        g_string_free(rest_process, TRUE);
+    }
+
+    process_data_free(data);
+}
+
+void
+milter_client_set_n_processing_sessions (MilterClient *client,
+                                         guint n_processing_sessions)
+{
+    MilterClientPrivate *priv;
+
+    priv = MILTER_CLIENT_GET_PRIVATE(client);
+    priv->n_processing_sessions = n_processing_sessions;
+}
+
+void
+milter_client_set_n_processed_sessions (MilterClient *client,
+                                        guint n_processed_sessions)
+{
+    MilterClientPrivate *priv;
+
+    priv = MILTER_CLIENT_GET_PRIVATE(client);
+    priv->n_processed_sessions = n_processed_sessions;
+}
+
+gboolean
+milter_client_need_maintain (MilterClient *client, guint n_finished_sessions)
+{
+    MilterClientPrivate *priv;
+    guint maintenance_interval, n_finished_sessions_in_interval;
+
+    priv = MILTER_CLIENT_GET_PRIVATE(client);
+    if (priv->n_processing_sessions == 0 && n_finished_sessions > 0)
+        return TRUE;
+
+    maintenance_interval = milter_client_get_maintenance_interval(client);
+    if (maintenance_interval == 0)
+        return FALSE;
+
+    n_finished_sessions_in_interval =
+        priv->n_processed_sessions % maintenance_interval;
+    return n_finished_sessions_in_interval < n_finished_sessions;
+}
+
+static void
+dispose_finished_data (MilterClient *client)
+{
+    MilterClientPrivate *priv;
+    guint n_processed_sessions_before, n_finished_sessions;
+
+    priv = MILTER_CLIENT_GET_PRIVATE(client);
+    if (!priv->finished_data)
+        return;
+
+    n_processed_sessions_before = priv->n_processed_sessions;
+    g_list_foreach(priv->finished_data, (GFunc)finish_processing, NULL);
+    g_list_free(priv->finished_data);
+    priv->finished_data = NULL;
+    n_finished_sessions =
+        priv->n_processed_sessions - n_processed_sessions_before;
+    g_signal_emit(client, signals[SESSIONS_FINISHED], 0, n_finished_sessions);
+
+    milter_statistics("[sessions][finished] %u(+%u) %u",
+                      priv->n_processed_sessions,
+                      n_finished_sessions,
+                      priv->n_processing_sessions);
+    if (milter_client_need_maintain(client, n_finished_sessions)) {
+        g_signal_emit(client, signals[MAINTAIN], 0);
+    }
+}
+
 static void
 dispose (GObject *object)
 {
@@ -268,6 +451,11 @@ dispose (GObject *object)
         priv->default_unix_socket_group = NULL;
     }
 
+    if (priv->worker_threads) {
+        g_thread_pool_free(priv->worker_threads, TRUE, FALSE);
+        priv->worker_threads = NULL;
+    }
+
     dispose_address(priv);
 
     if (priv->effective_user) {
@@ -280,6 +468,9 @@ dispose (GObject *object)
         priv->effective_group = NULL;
     }
 
+    dispose_finisher(priv);
+    dispose_finished_data(MILTER_CLIENT(object));
+
     G_OBJECT_CLASS(_milter_client_parent_class)->dispose(object);
 }
 
@@ -466,67 +657,44 @@ milter_client_set_listen_channel (MilterClient *client, GIOChannel *channel)
 }
 
 static gboolean
-cb_idle_free_data (gpointer _data)
+single_worker_finisher (gpointer _data)
 {
     MilterClientProcessData *data = _data;
-    GList *processing_data, *process_data;
-    guint n_connections;
-    GString *rest_process;
-    guint tag;
 
-    tag = milter_agent_get_tag(MILTER_AGENT(data->context));
-    milter_debug("[%u] [client][finish]", tag);
-    data->priv->processing_data =
-        g_list_remove(data->priv->processing_data, data);
-    data->priv->n_connections--;
+    milter_debug("[client][finisher][run]");
 
-    processing_data = g_list_copy(data->priv->processing_data);
-
-    n_connections = data->priv->n_connections;
-    g_mutex_lock(data->priv->quit_mutex);
-    if (data->priv->quitting && n_connections == 0) {
-        milter_debug("[%u] [client][loop][quit]", tag);
-        g_main_loop_quit(data->priv->main_loop);
-    }
-    g_mutex_unlock(data->priv->quit_mutex);
-
-    process_data_free(data);
-
-    rest_process = g_string_new("[");
-    for (process_data = processing_data;
-         process_data;
-         process_data = g_list_next(process_data)) {
-        MilterClientProcessData *_process_data = process_data->data;
-        g_string_append_printf(
-            rest_process, "<%u>, ",
-            milter_agent_get_tag(MILTER_AGENT(_process_data->context)));
-    }
-    if (processing_data)
-        g_string_truncate(rest_process, rest_process->len - 2);
-    g_string_append(rest_process, "]");
-    g_list_free(processing_data);
-    milter_debug("[%u] [client][rest] %s", tag, rest_process->str);
-    g_string_free(rest_process, TRUE);
+    data->priv->finisher_id = 0;
+    dispose_finished_data(data->client);
 
     return FALSE;
 }
 
 static void
-cb_finished (MilterClientContext *context, gpointer _data)
+single_worker_cb_finished (MilterClientContext *context, gpointer _data)
 {
+    MilterClientPrivate *priv;
     MilterClientProcessData *data = _data;
 
-    g_idle_add(cb_idle_free_data, data);
+    dispose_process_data_finished_handler(data);
+    priv = data->priv;
+    priv->finished_data = g_list_prepend(priv->finished_data, data);
+    if (priv->finisher_id == 0) {
+        priv->finisher_id = g_idle_add_full(G_PRIORITY_DEFAULT,
+                                            single_worker_finisher,
+                                            data,
+                                            NULL);
+    }
 }
 
 typedef struct _ClientChannelSetupData
 {
     MilterClient *client;
     GIOChannel *channel;
+    MilterGenericSocketAddress address;
 } ClientChannelSetupData;
 
 static gboolean
-cb_timeout_client_channel_setup (gpointer user_data)
+single_worker_cb_idle_client_channel_setup (gpointer user_data)
 {
     ClientChannelSetupData *setup_data = user_data;
     MilterClient *client;
@@ -539,8 +707,9 @@ cb_timeout_client_channel_setup (gpointer user_data)
     client = setup_data->client;
     priv = MILTER_CLIENT_GET_PRIVATE(client);
 
+    context = milter_client_context_new(client);
+
     writer = milter_writer_io_channel_new(setup_data->channel);
-    context = milter_client_context_new();
     milter_agent_set_writer(MILTER_AGENT(context), writer);
     g_object_unref(writer);
 
@@ -550,6 +719,8 @@ cb_timeout_client_channel_setup (gpointer user_data)
 
     milter_client_context_set_timeout(context, priv->timeout);
 
+    milter_client_context_set_socket_address(context, &(setup_data->address));
+
     data = g_new(MilterClientProcessData, 1);
     data->priv = priv;
     data->client = client;
@@ -558,12 +729,14 @@ cb_timeout_client_channel_setup (gpointer user_data)
     milter_debug("[%u] [client][start]",
                  milter_agent_get_tag(MILTER_AGENT(data->context)));
 
-    g_signal_connect(context, "finished", G_CALLBACK(cb_finished), data);
+    data->finished_handler_id =
+        g_signal_connect(context, "finished",
+                         G_CALLBACK(single_worker_cb_finished), data);
 
     priv->processing_data = g_list_prepend(priv->processing_data, data);
 
+    milter_agent_start(MILTER_AGENT(context), NULL);
     g_signal_emit(client, signals[CONNECTION_ESTABLISHED], 0, context);
-    milter_agent_start(MILTER_AGENT(context));
 
     g_io_channel_unref(setup_data->channel);
     g_free(setup_data);
@@ -572,27 +745,32 @@ cb_timeout_client_channel_setup (gpointer user_data)
 }
 
 static void
-process_client_channel (MilterClient *client, GIOChannel *channel)
+single_worker_process_client_channel (MilterClient *client, GIOChannel *channel,
+                                      MilterGenericSocketAddress *address,
+                                      socklen_t address_size)
 {
     ClientChannelSetupData *data;
 
     data = g_new(ClientChannelSetupData, 1);
     data->client = client;
     data->channel = channel;
+    memcpy(&(data->address), address, address_size);
     g_io_channel_ref(channel);
 
-    g_timeout_add(0, cb_timeout_client_channel_setup, data);
+    g_idle_add_full(G_PRIORITY_DEFAULT,
+                    single_worker_cb_idle_client_channel_setup,
+                    data,
+                    NULL);
 }
 
 static gboolean
-accept_client (gint server_fd, MilterClient *client)
+accept_connection (MilterClient *client, gint server_fd,
+                   GIOChannel **client_channel,
+                   MilterGenericSocketAddress *address,
+                   socklen_t *address_size)
 {
     MilterClientPrivate *priv;
     gint client_fd;
-    GIOChannel *client_channel;
-    MilterGenericSocketAddress address;
-    gchar *spec;
-    socklen_t address_size;
     guint n_suspend, suspend_time, max_connections;
 
     priv = MILTER_CLIENT_GET_PRIVATE(client);
@@ -600,12 +778,12 @@ accept_client (gint server_fd, MilterClient *client)
     suspend_time = milter_client_get_suspend_time_on_unacceptable(client);
     max_connections = milter_client_get_max_connections(client);
     for (n_suspend = 0;
-         0 < max_connections && max_connections <= priv->n_connections;
+         0 < max_connections && max_connections <= priv->n_processing_sessions;
          n_suspend++) {
         milter_warning("[client][accept][suspend] "
                        "too many processing connection: %u, max: %u; "
                        "suspend accepting connection in %d seconds: #%u",
-                       priv->n_connections,
+                       priv->n_processing_sessions,
                        max_connections,
                        suspend_time,
                        n_suspend);
@@ -614,9 +792,9 @@ accept_client (gint server_fd, MilterClient *client)
                        "resume accepting connection: #%u", n_suspend);
     }
 
-    address_size = sizeof(address);
-    memset(&address, '\0', address_size);
-    client_fd = accept(server_fd, (struct sockaddr *)(&address), &address_size);
+    *address_size = sizeof(*address);
+    memset(address, '\0', *address_size);
+    client_fd = accept(server_fd, (struct sockaddr *)(address), address_size);
     if (client_fd == -1) {
         GError *error = NULL;
         g_set_error(&error,
@@ -638,35 +816,58 @@ accept_client (gint server_fd, MilterClient *client)
                            "resume accepting connection.");
         }
 
-        return TRUE;
+        return FALSE;
     }
 
-    priv->n_connections++;
-    spec = milter_connection_address_to_spec(&(address.address.base));
-    milter_debug("[client][accept] %d:%s", client_fd, spec);
-    g_free(spec);
+    priv->n_processing_sessions++;
+    if (milter_need_debug_log()) {
+        gchar *spec;
+        spec = milter_connection_address_to_spec(&(address->address.base));
+        milter_debug("[client][accept] %d:%s", client_fd, spec);
+        g_free(spec);
+    }
 
-    client_channel = g_io_channel_unix_new(client_fd);
-    g_io_channel_set_encoding(client_channel, NULL, NULL);
-    g_io_channel_set_flags(client_channel, G_IO_FLAG_NONBLOCK, NULL);
-    g_io_channel_set_close_on_unref(client_channel, TRUE);
-    process_client_channel(client, client_channel);
-    g_io_channel_unref(client_channel);
+    *client_channel = g_io_channel_unix_new(client_fd);
+    g_io_channel_set_encoding(*client_channel, NULL, NULL);
+    g_io_channel_set_flags(*client_channel, G_IO_FLAG_NONBLOCK, NULL);
+    g_io_channel_set_close_on_unref(*client_channel, TRUE);
 
     return TRUE;
 }
 
 static gboolean
-server_watch_func (GIOChannel *channel, GIOCondition condition, gpointer data)
+single_worker_accept_connection (MilterClient *client, gint server_fd)
+{
+    gboolean accepted;
+    GIOChannel *client_channel;
+    MilterGenericSocketAddress address;
+    socklen_t address_size;
+
+    accepted = accept_connection(client, server_fd, &client_channel,
+                                 &address, &address_size);
+    if (accepted) {
+        single_worker_process_client_channel(client, client_channel,
+                                             &address, address_size);
+        g_io_channel_unref(client_channel);
+    }
+
+    return TRUE;
+}
+
+static gboolean
+cb_server_status_changed (MilterClient *client,
+                          GIOChannel *channel, GIOCondition condition,
+                          AcceptConnectionFunction accept_connection)
 {
-    MilterClient *client = data;
     MilterClientPrivate *priv;
     gboolean keep_callback = TRUE;
 
     priv = MILTER_CLIENT_GET_PRIVATE(client);
     if (condition & G_IO_IN ||
         condition & G_IO_PRI) {
-        keep_callback = accept_client(g_io_channel_unix_get_fd(channel), client);
+        gint fd;
+        fd = g_io_channel_unix_get_fd(channel);
+        keep_callback = accept_connection(client, fd);
     }
 
     if (condition & G_IO_ERR ||
@@ -695,7 +896,17 @@ server_watch_func (GIOChannel *channel, GIOCondition condition, gpointer data)
 }
 
 static gboolean
-cb_timeout_unlock_quit_mutex (gpointer data)
+single_worker_server_watch_func (GIOChannel *channel, GIOCondition condition,
+                                 gpointer data)
+{
+    MilterClient *client = data;
+
+    return cb_server_status_changed(client, channel, condition,
+                                    single_worker_accept_connection);
+}
+
+static gboolean
+single_worker_cb_idle_unlock_quit_mutex (gpointer data)
 {
     MilterClientPrivate *priv;
 
@@ -707,20 +918,21 @@ cb_timeout_unlock_quit_mutex (gpointer data)
 }
 
 static gpointer
-server_accept_thread (gpointer data)
+single_worker_accept_thread (gpointer data)
 {
     MilterClient *client = data;
     MilterClientPrivate *priv;
-    GSource *timeout_source;
+    GSource *source;
 
     priv = MILTER_CLIENT_GET_PRIVATE(client);
 
-    timeout_source = g_timeout_source_new(0);
-    g_source_set_callback(timeout_source,
-                          cb_timeout_unlock_quit_mutex,
+    source = g_idle_source_new();
+    g_source_set_priority(source, G_PRIORITY_DEFAULT);
+    g_source_set_callback(source,
+                          single_worker_cb_idle_unlock_quit_mutex,
                           client, NULL);
-    g_source_attach(timeout_source, g_main_loop_get_context(priv->accept_loop));
-    g_source_unref(timeout_source);
+    g_source_attach(source, g_main_loop_get_context(priv->accept_loop));
+    g_source_unref(source);
 
     g_mutex_lock(priv->quit_mutex);
     if (priv->quitting)
@@ -731,6 +943,186 @@ server_accept_thread (gpointer data)
     return NULL;
 }
 
+static gboolean
+single_worker_start_accept (MilterClient *client)
+{
+    MilterClientPrivate *priv;
+    GThread *thread;
+
+    priv = MILTER_CLIENT_GET_PRIVATE(client);
+    priv->main_loop = g_main_loop_new(NULL, FALSE);
+    thread = g_thread_create(single_worker_accept_thread, client, TRUE, NULL);
+
+    g_mutex_lock(priv->quit_mutex);
+    if (priv->quitting) {
+        g_mutex_unlock(priv->quit_mutex);
+    } else {
+        g_idle_add_full(G_PRIORITY_DEFAULT,
+                        single_worker_cb_idle_unlock_quit_mutex,
+                        client,
+                        NULL);
+        g_main_loop_run(priv->main_loop);
+    }
+    g_thread_join(thread);
+    g_main_loop_unref(priv->main_loop);
+    priv->main_loop = NULL;
+
+    return TRUE;
+}
+
+static void
+multi_workers_process_client_channel (MilterClient *client, GIOChannel *channel,
+                                      MilterGenericSocketAddress *address,
+                                      socklen_t address_size)
+{
+    MilterClientPrivate *priv;
+    MilterClientContext *context;
+    MilterClientProcessData *data;
+    MilterWriter *writer;
+    MilterReader *reader;
+    GError *error = NULL;
+
+    priv = MILTER_CLIENT_GET_PRIVATE(client);
+
+    context = milter_client_context_new(client);
+
+    writer = milter_writer_io_channel_new(channel);
+    milter_agent_set_writer(MILTER_AGENT(context), writer);
+    g_object_unref(writer);
+
+    reader = milter_reader_io_channel_new(channel);
+    milter_agent_set_reader(MILTER_AGENT(context), reader);
+    g_object_unref(reader);
+
+    milter_client_context_set_timeout(context, priv->timeout);
+
+    milter_client_context_set_socket_address(context, address);
+
+    data = g_new(MilterClientProcessData, 1);
+    data->priv = priv;
+    data->client = client;
+    data->context = context;
+    data->finished_handler_id = 0;
+
+    priv->processing_data = g_list_prepend(priv->processing_data, data);
+    g_thread_pool_push(priv->worker_threads, data, &error);
+    if (error) {
+        GError *client_error;
+
+        client_error = g_error_new(MILTER_CLIENT_ERROR,
+                                   MILTER_CLIENT_ERROR_THREAD,
+                                   "[client][error][thread] "
+                                   "failed to push a data to thread pool: %s",
+                                   error->message);
+        g_error_free(error);
+        milter_error_emittable_emit(MILTER_ERROR_EMITTABLE(client),
+                                    client_error);
+        g_error_free(client_error);
+
+        process_data_free(data);
+    }
+}
+
+static gboolean
+multi_workers_accept_connection (MilterClient *client, gint server_fd)
+{
+    MilterClientPrivate *priv;
+    gboolean accepted;
+    GIOChannel *client_channel;
+    MilterGenericSocketAddress address;
+    socklen_t address_size;
+
+    priv = MILTER_CLIENT_GET_PRIVATE(client);
+    accepted = accept_connection(client, server_fd, &client_channel,
+                                 &address, &address_size);
+    if (accepted) {
+        multi_workers_process_client_channel(client, client_channel,
+                                             &address, address_size);
+        g_io_channel_unref(client_channel);
+    }
+
+    return TRUE;
+}
+
+static gboolean
+multi_workers_server_watch_func (GIOChannel *channel, GIOCondition condition,
+                                 gpointer data)
+{
+    MilterClient *client = data;
+
+    return cb_server_status_changed(client, channel, condition,
+                                    multi_workers_accept_connection);
+}
+
+static void
+multi_workers_cb_finished (MilterClientContext *context, gpointer _data)
+{
+    MilterClientProcessData *data = _data;
+
+    finish_processing(data);
+}
+
+static void
+multi_workers_process_client_channel_thread (gpointer data_, gpointer user_data)
+{
+    MilterClient *client = data_;
+    MilterClientProcessData *data = user_data;
+    GMainContext *main_context;
+    GMainLoop *main_loop;
+
+    main_context = g_main_context_new();
+    main_loop = g_main_loop_new(main_context, FALSE);
+
+    milter_debug("[%u] [client][start]",
+                 milter_agent_get_tag(MILTER_AGENT(data->context)));
+
+    data->finished_handler_id =
+        g_signal_connect(data->context, "finished",
+                         G_CALLBACK(multi_workers_cb_finished), data);
+
+    milter_agent_start(MILTER_AGENT(data->context), main_context);
+    g_signal_emit(client, signals[CONNECTION_ESTABLISHED], 0, data->context);
+
+    g_main_loop_run(main_loop);
+    g_main_context_unref(main_context);
+}
+
+static gboolean
+multi_workers_start_accept (MilterClient *client)
+{
+    MilterClientPrivate *priv;
+    gint max_threads = 10;
+    GError *error = NULL;
+
+    priv = MILTER_CLIENT_GET_PRIVATE(client);
+    priv->worker_threads =
+        g_thread_pool_new(multi_workers_process_client_channel_thread,
+                          client,
+                          max_threads,
+                          FALSE,
+                          &error);
+    if (!priv->worker_threads) {
+        GError *client_error;
+        client_error = g_error_new(MILTER_CLIENT_ERROR,
+                                   MILTER_CLIENT_ERROR_THREAD,
+                                   "[client][error][accept] "
+                                   "failed to create a thread pool "
+                                   "for processing accepted connection: %s",
+                                   error->message);
+        g_error_free(error);
+        milter_error_emittable_emit(MILTER_ERROR_EMITTABLE(client),
+                                    client_error);
+        g_error_free(client_error);
+        return FALSE;
+    }
+
+    g_main_loop_run(priv->accept_loop);
+
+    g_thread_pool_free(priv->worker_threads, TRUE, TRUE);
+    priv->worker_threads = NULL;
+
+    return TRUE;
+}
 
 
 static GIOChannel *
@@ -920,87 +1312,11 @@ milter_client_drop_privilege (MilterClient *client, GError **error)
     return switch_group(client, error) && switch_user(client, error);
 }
 
-static gboolean
-detach_io (MilterClient *client, GError **error)
-{
-    gboolean success = FALSE;
-    int null_stdin_fd = -1;
-    int null_stdout_fd = -1;
-    int null_stderr_fd = -1;
-
-    null_stdin_fd = open("/dev/null", O_RDONLY);
-    if (null_stdin_fd == -1) {
-        g_set_error(error,
-                    MILTER_CLIENT_ERROR,
-                    MILTER_CLIENT_ERROR_DETACH_IO,
-                    "failed to open /dev/null for STDIN: %s",
-                    g_strerror(errno));
-        goto cleanup;
-    }
-
-    null_stdout_fd = open("/dev/null", O_WRONLY);
-    if (null_stdout_fd == -1) {
-        g_set_error(error,
-                    MILTER_CLIENT_ERROR,
-                    MILTER_CLIENT_ERROR_DETACH_IO,
-                    "failed to open /dev/null for STDOUT: %s",
-                    g_strerror(errno));
-        goto cleanup;
-    }
-
-    null_stderr_fd = open("/dev/null", O_WRONLY);
-    if (null_stderr_fd == -1) {
-        g_set_error(error,
-                    MILTER_CLIENT_ERROR,
-                    MILTER_CLIENT_ERROR_DETACH_IO,
-                    "failed to open /dev/null for STDERR: %s",
-                    g_strerror(errno));
-        goto cleanup;
-    }
-
-    if (dup2(null_stdin_fd, STDIN_FILENO) == -1) {
-        g_set_error(error,
-                    MILTER_CLIENT_ERROR,
-                    MILTER_CLIENT_ERROR_DETACH_IO,
-                    "failed to detach STDIN: %s",
-                    g_strerror(errno));
-        goto cleanup;
-    }
-
-    if (dup2(null_stdout_fd, STDOUT_FILENO) == -1) {
-        g_set_error(error,
-                    MILTER_CLIENT_ERROR,
-                    MILTER_CLIENT_ERROR_DETACH_IO,
-                    "failed to detach STDOUT: %s",
-                    g_strerror(errno));
-        goto cleanup;
-    }
-
-    if (dup2(null_stderr_fd, STDERR_FILENO) == -1) {
-        g_set_error(error,
-                    MILTER_CLIENT_ERROR,
-                    MILTER_CLIENT_ERROR_DETACH_IO,
-                    "failed to detach STDERR: %s",
-                    g_strerror(errno));
-        goto cleanup;
-    }
-
-    success = TRUE;
-
-cleanup:
-    if (null_stdin_fd == -1)
-        close(null_stdin_fd);
-    if (null_stdout_fd == -1)
-        close(null_stdout_fd);
-    if (null_stderr_fd == -1)
-        close(null_stderr_fd);
-
-    return success;
-}
-
 gboolean
 milter_client_daemonize (MilterClient *client, GError **error)
 {
+    gchar *error_message = NULL;
+
     switch (fork()) {
     case 0:
         break;
@@ -1049,8 +1365,15 @@ milter_client_daemonize (MilterClient *client, GError **error)
         return FALSE;
     }
 
-    if (!detach_io(client, error))
+    if (!milter_utils_detach_io(&error_message)) {
+        g_set_error(error,
+                    MILTER_CLIENT_ERROR,
+                    MILTER_CLIENT_ERROR_DETACH_IO,
+                    "%s",
+                    error_message);
+        g_free(error_message);
         return FALSE;
+    }
 
     return TRUE;
 }
@@ -1061,11 +1384,11 @@ milter_client_main (MilterClient *client)
     MilterClientPrivate *priv;
     GError *error = NULL;
     GSource *watch_source;
-    GThread *thread;
+    gboolean success;
 
     priv = MILTER_CLIENT_GET_PRIVATE(client);
 
-    if (priv->listening_channel || priv->n_connections > 0) {
+    if (priv->listening_channel || priv->n_processing_sessions > 0) {
         GError *error;
 
         error = g_error_new(MILTER_CLIENT_ERROR,
@@ -1097,23 +1420,25 @@ milter_client_main (MilterClient *client)
     watch_source = g_io_create_watch(priv->listening_channel,
                                      G_IO_IN | G_IO_PRI |
                                      G_IO_ERR | G_IO_HUP | G_IO_NVAL);
-    g_source_set_callback(watch_source, (GSourceFunc)server_watch_func,
-                          client, NULL);
+    if (priv->multi_workers_mode) {
+        g_source_set_callback(watch_source,
+                              (GSourceFunc)multi_workers_server_watch_func,
+                              client, NULL);
+    } else {
+        g_source_set_callback(watch_source,
+                              (GSourceFunc)single_worker_server_watch_func,
+                              client, NULL);
+    }
     priv->server_watch_id =
         g_source_attach(watch_source,
                         g_main_loop_get_context(priv->accept_loop));
     g_source_unref(watch_source);
 
-    thread = g_thread_create(server_accept_thread, client, TRUE, NULL);
-
-    g_mutex_lock(priv->quit_mutex);
-    if (priv->quitting) {
-        g_mutex_unlock(priv->quit_mutex);
+    if (priv->multi_workers_mode) {
+        success = multi_workers_start_accept(client);
     } else {
-        g_timeout_add(0, cb_timeout_unlock_quit_mutex, client);
-        g_main_loop_run(priv->main_loop);
+        success = single_worker_start_accept(client);
     }
-    g_thread_join(thread);
 
     if (priv->listening_channel) {
         g_io_channel_unref(priv->listening_channel);
@@ -1162,7 +1487,7 @@ milter_client_shutdown (MilterClient *client)
         g_io_channel_unref(priv->listening_channel);
         priv->listening_channel = NULL;
 
-        if (priv->n_connections == 0)
+        if (priv->n_processing_sessions == 0)
             g_main_loop_quit(priv->main_loop);
     }
     g_mutex_unlock(priv->quit_mutex);
@@ -1381,6 +1706,50 @@ milter_client_set_effective_group (MilterClient *client,
     }
 }
 
+guint
+milter_client_get_maintenance_interval (MilterClient *client)
+{
+    MilterClientClass *klass;
+
+    klass = MILTER_CLIENT_GET_CLASS(client);
+    if (klass->get_maintenance_interval)
+        return klass->get_maintenance_interval(client);
+    else
+        return MILTER_CLIENT_GET_PRIVATE(client)->maintenance_interval;
+}
+
+void
+milter_client_set_maintenance_interval (MilterClient *client, guint n_sessions)
+{
+    MilterClientClass *klass;
+
+    klass = MILTER_CLIENT_GET_CLASS(client);
+    if (klass->set_maintenance_interval)
+        klass->set_maintenance_interval(client, n_sessions);
+    else
+        MILTER_CLIENT_GET_PRIVATE(client)->maintenance_interval = n_sessions;
+}
+
+void
+milter_client_processing_context_foreach (MilterClient *client,
+                                          GFunc func, gpointer user_data)
+{
+    MilterClientPrivate *priv;
+    GList *node;
+
+    priv = MILTER_CLIENT_GET_PRIVATE(client);
+    for (node = priv->processing_data; node; node = g_list_next(node)) {
+        MilterClientProcessData *data = node->data;
+        func(data->context, user_data);
+    }
+}
+
+guint
+milter_client_get_n_processing_sessions (MilterClient *client)
+{
+    return MILTER_CLIENT_GET_PRIVATE(client)->n_processing_sessions;
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
