@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -82,13 +82,16 @@ gboolean             milter_agent_write_packet      (MilterAgent *agent,
 gboolean             milter_agent_flush             (MilterAgent *agent,
                                                      GError **error);
 
-void                 milter_agent_start             (MilterAgent *agent);
+void                 milter_agent_start             (MilterAgent *agent,
+                                                     GMainContext *context);
 void                 milter_agent_shutdown          (MilterAgent *agent);
 
 guint                milter_agent_get_tag           (MilterAgent *agent);
 void                 milter_agent_set_tag           (MilterAgent *agent,
                                                      guint        tag);
 
+gdouble              milter_agent_get_elapsed       (MilterAgent *agent);
+
 G_END_DECLS
 
 #endif /* __MILTER_AGENT_H__ */
