@@ -25,6 +25,7 @@
 #include "milter-connection.h"
 #include "milter-esmtp.h"
 #include "milter-message-result.h"
+#include "milter-session-result.h"
 
 /* enumerations from "milter-protocol.h" */
 GType
@@ -299,6 +300,7 @@ milter_writer_error_get_type (void)
     static const GEnumValue values[] = {
       { MILTER_WRITER_ERROR_NO_CHANNEL, "MILTER_WRITER_ERROR_NO_CHANNEL", "no-channel" },
       { MILTER_WRITER_ERROR_IO_ERROR, "MILTER_WRITER_ERROR_IO_ERROR", "io-error" },
+      { MILTER_WRITER_ERROR_NOT_READY, "MILTER_WRITER_ERROR_NOT_READY", "not-ready" },
       { 0, NULL, NULL }
     };
     etype = g_enum_register_static ("MilterWriterError", values);
@@ -311,6 +313,8 @@ milter_log_level_flags_get_type (void)
   static GType etype = 0;
   if (etype == 0) {
     static const GFlagsValue values[] = {
+      { MILTER_LOG_LEVEL_DEFAULT, "MILTER_LOG_LEVEL_DEFAULT", "default" },
+      { MILTER_LOG_LEVEL_NONE, "MILTER_LOG_LEVEL_NONE", "none" },
       { MILTER_LOG_LEVEL_CRITICAL, "MILTER_LOG_LEVEL_CRITICAL", "critical" },
       { MILTER_LOG_LEVEL_ERROR, "MILTER_LOG_LEVEL_ERROR", "error" },
       { MILTER_LOG_LEVEL_WARNING, "MILTER_LOG_LEVEL_WARNING", "warning" },
@@ -331,6 +335,7 @@ milter_log_item_flags_get_type (void)
   if (etype == 0) {
     static const GFlagsValue values[] = {
       { MILTER_LOG_ITEM_DEFAULT, "MILTER_LOG_ITEM_DEFAULT", "default" },
+      { MILTER_LOG_ITEM_NONE, "MILTER_LOG_ITEM_NONE", "none" },
       { MILTER_LOG_ITEM_DOMAIN, "MILTER_LOG_ITEM_DOMAIN", "domain" },
       { MILTER_LOG_ITEM_LEVEL, "MILTER_LOG_ITEM_LEVEL", "level" },
       { MILTER_LOG_ITEM_LOCATION, "MILTER_LOG_ITEM_LOCATION", "location" },
