@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -23,7 +23,6 @@
 #include <glib-object.h>
 
 #include <milter/manager/milter-manager-objects.h>
-#include <milter/manager/milter-manager-configuration.h>
 #include <milter/manager/milter-manager-child.h>
 #include <milter/core/milter-reply-signals.h>
 
@@ -42,7 +41,6 @@ typedef enum
 {
     MILTER_MANAGER_CHILDREN_ERROR_MILTER_EXIT,
     MILTER_MANAGER_CHILDREN_ERROR_NO_NEGOTIATION,
-    MILTER_MANAGER_CHILDREN_ERROR_NO_NEGOTIATION_RESPONSE,
     MILTER_MANAGER_CHILDREN_ERROR_NO_DEMAND_COMMAND,
     MILTER_MANAGER_CHILDREN_ERROR_NO_ALIVE_MILTER,
     MILTER_MANAGER_CHILDREN_ERROR_NO_MESSAGE_PROCESSING_MILTER
@@ -137,6 +135,16 @@ void                   milter_manager_children_set_tag     (MilterManagerChildre
                                                             guint                  tag);
 
 
+gboolean               milter_manager_children_get_smtp_client_address
+                                                           (MilterManagerChildren *children,
+                                                            struct sockaddr       **address,
+                                                            socklen_t             *address_length);
+
+
+MilterOption          *milter_manager_children_get_option
+                                          (MilterManagerChildren *children);
+
+
 #endif /* __MILTER_MANAGER_CHILDREN_H__ */
 
 /*
