@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -62,11 +62,16 @@ struct _MilterManagerConfigurationClass
     gboolean     (*load_custom)   (MilterManagerConfiguration *configuration,
                                    const gchar                *file_name,
                                    GError                    **error);
-    void         (*maintain)      (MilterManagerConfiguration *configuration);
+    gboolean     (*maintain)      (MilterManagerConfiguration *configuration,
+                                   GError                    **error);
+    void         (*connected)     (MilterManagerConfiguration *configuration,
+                                   MilterManagerLeader        *leader);
     void         (*to_xml)        (MilterManagerConfiguration *configuration,
                                    GString                    *xml,
                                    guint                       indent);
     gchar       *(*dump)          (MilterManagerConfiguration *configuration);
+    gboolean     (*clear)         (MilterManagerConfiguration *configuration,
+                                   GError                    **error);
 };
 
 GQuark        milter_manager_configuration_error_quark (void);
@@ -216,6 +221,8 @@ guint         milter_manager_configuration_get_maintenance_interval
 void          milter_manager_configuration_set_maintenance_interval
                                      (MilterManagerConfiguration *configuration,
                                       guint                       n_sessions);
+void          milter_manager_configuration_maintain
+                                     (MilterManagerConfiguration *configuration);
 
 guint         milter_manager_configuration_get_suspend_time_on_unacceptable
                                      (MilterManagerConfiguration *configuration);
@@ -241,6 +248,9 @@ void          milter_manager_configuration_set_custom_configuration_directory
                                      (MilterManagerConfiguration *configuration,
                                       const gchar                *directory);
 
+void          milter_manager_configuration_clear_signal_handlers
+                                     (MilterManagerConfiguration *configuration);
+
 void          milter_manager_configuration_add_egg
                                      (MilterManagerConfiguration *configuration,
                                       MilterManagerEgg *egg);
@@ -258,8 +268,10 @@ void          milter_manager_configuration_remove_egg_by_name
                                       const gchar *name);
 void          milter_manager_configuration_clear_eggs
                                      (MilterManagerConfiguration *configuration);
-MilterManagerChildren *milter_manager_configuration_create_children
-                                     (MilterManagerConfiguration *configuration);
+void          milter_manager_configuration_setup_children
+                                     (MilterManagerConfiguration *configuration,
+                                      MilterManagerChildren      *children,
+                                      MilterClientContext        *context);
 void          milter_manager_configuration_add_applicable_condition
                                      (MilterManagerConfiguration *configuration,
                                       MilterManagerApplicableCondition *condition);
@@ -283,6 +295,9 @@ void          milter_manager_configuration_clear
 
 void          milter_manager_configuration_session_finished
                                      (MilterManagerConfiguration *configuration);
+void          milter_manager_configuration_n_sessions_finished
+                                     (MilterManagerConfiguration *configuration,
+                                      guint                       n_sessions);
 
 
 gchar        *milter_manager_configuration_to_xml
@@ -295,6 +310,25 @@ void          milter_manager_configuration_to_xml_string
 gchar        *milter_manager_configuration_dump
                                      (MilterManagerConfiguration *configuration);
 
+void          milter_manager_configuration_set_location
+                                    (MilterManagerConfiguration *configuration,
+                                     const gchar                *key,
+                                     const gchar                *file,
+                                     gint                        line);
+void          milter_manager_configuration_reset_location
+                                    (MilterManagerConfiguration *configuration,
+                                     const gchar                *key);
+gconstpointer milter_manager_configuration_get_location
+                                    (MilterManagerConfiguration *configuration,
+                                     const gchar                *key);
+GHashTable   *milter_manager_configuration_get_locations
+                                    (MilterManagerConfiguration *configuration);
+guint         milter_manager_configuration_get_connection_check_interval
+                                     (MilterManagerConfiguration *configuration);
+void          milter_manager_configuration_set_connection_check_interval
+                                     (MilterManagerConfiguration *configuration,
+                                      guint                       interval_in_seconds);
+
 G_END_DECLS
 
 #endif /* __MILTER_MANAGER_CONFIGURATION_H__ */
