@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -21,6 +21,7 @@
 #  include "../../config.h"
 #endif /* HAVE_CONFIG_H */
 
+#include <milter/core/milter-marshalers.h>
 #include "milter-manager-leader.h"
 #include "milter-manager-enum-types.h"
 #include "milter-manager-children.h"
@@ -40,17 +41,32 @@ struct _MilterManagerLeaderPrivate
     gboolean sent_end_of_message;
     GIOChannel *launcher_read_channel;
     GIOChannel *launcher_write_channel;
+    gboolean processing;
+    gboolean requesting_to_children;
+    guint tag;
+};
+
+enum
+{
+    CONNECTION_CHECK,
+    LAST_SIGNAL
 };
 
 enum
 {
     PROP_0,
     PROP_CONFIGURATION,
-    PROP_CLIENT_CONTEXT
+    PROP_CLIENT_CONTEXT,
+    PROP_CHILDREN
 };
 
+static gint signals[LAST_SIGNAL] = {0};
+
+static void         finished           (MilterFinishedEmittable *emittable);
+
 MILTER_IMPLEMENT_ERROR_EMITTABLE(error_emittable_init);
-MILTER_IMPLEMENT_FINISHED_EMITTABLE(finished_emittable_init);
+MILTER_IMPLEMENT_FINISHED_EMITTABLE_WITH_CODE(finished_emittable_init,
+                                              iface->finished = finished)
 G_DEFINE_TYPE_WITH_CODE(MilterManagerLeader, milter_manager_leader, G_TYPE_OBJECT,
     G_IMPLEMENT_INTERFACE(MILTER_TYPE_ERROR_EMITTABLE, error_emittable_init)
     G_IMPLEMENT_INTERFACE(MILTER_TYPE_FINISHED_EMITTABLE, finished_emittable_init))
@@ -64,9 +80,18 @@ static void get_property   (GObject         *object,
                             guint            prop_id,
                             GValue          *value,
                             GParamSpec      *pspec);
+static gboolean connection_check_default
+                           (MilterManagerLeader *leader);
 static void teardown_children_signals
                            (MilterManagerLeader *leader,
                             MilterManagerChildren *children);
+static void reply          (MilterManagerLeader *leader,
+                            MilterStatus         status);
+static gboolean boolean_handled_accumulator
+                           (GSignalInvocationHint *hint,
+                            GValue                *return_accumulator,
+                            const GValue          *handler_return,
+                            gpointer               data);
 
 static void
 milter_manager_leader_class_init (MilterManagerLeaderClass *klass)
@@ -80,6 +105,8 @@ milter_manager_leader_class_init (MilterManagerLeaderClass *klass)
     gobject_class->set_property = set_property;
     gobject_class->get_property = get_property;
 
+    klass->connection_check = connection_check_default;
+
     spec = g_param_spec_object("configuration",
                                "Configuration",
                                "The configuration of the milter leader",
@@ -94,6 +121,22 @@ milter_manager_leader_class_init (MilterManagerLeaderClass *klass)
                                G_PARAM_READWRITE);
     g_object_class_install_property(gobject_class, PROP_CLIENT_CONTEXT, spec);
 
+    spec = g_param_spec_object("children",
+                               "Children",
+                               "The object manages child milters.",
+                               MILTER_TYPE_MANAGER_CHILDREN,
+                               G_PARAM_READABLE);
+    g_object_class_install_property(gobject_class, PROP_CHILDREN, spec);
+
+    signals[CONNECTION_CHECK] =
+        g_signal_new("connection-check",
+                     MILTER_TYPE_MANAGER_LEADER,
+                     G_SIGNAL_RUN_LAST,
+                     G_STRUCT_OFFSET(MilterManagerLeaderClass, connection_check),
+                     boolean_handled_accumulator, NULL,
+                     _milter_marshal_BOOLEAN__VOID,
+                     G_TYPE_BOOLEAN, 0, G_TYPE_NONE);
+
     g_type_class_add_private(gobject_class,
                              sizeof(MilterManagerLeaderPrivate));
 }
@@ -111,6 +154,96 @@ milter_manager_leader_init (MilterManagerLeader *leader)
     priv->sent_end_of_message = FALSE;
     priv->launcher_read_channel = NULL;
     priv->launcher_write_channel = NULL;
+    priv->processing = FALSE;
+    priv->requesting_to_children = FALSE;
+    priv->tag = 0;
+}
+
+gboolean
+milter_manager_leader_check_connection (MilterManagerLeader *leader)
+{
+    MilterManagerLeaderPrivate *priv;
+    gboolean connected = TRUE;
+    gboolean skip = FALSE;
+    gdouble elapsed = 0.0;
+    gchar *state_nick = NULL;
+
+    priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
+
+    if (milter_need_log(MILTER_LOG_LEVEL_DEBUG |
+                        MILTER_LOG_LEVEL_STATISTICS)) {
+        MilterAgent *agent;
+        agent = MILTER_AGENT(priv->client_context);
+        elapsed = milter_agent_get_elapsed(agent);
+    }
+
+    if (milter_need_debug_log()) {
+        state_nick =
+            milter_utils_get_enum_nick_name(MILTER_TYPE_MANAGER_LEADER_STATE,
+                                            priv->state);
+    }
+
+    if (!priv->processing) {
+        milter_debug("[%u] [leader][connection-check][%s][skip] "
+                     "not negotiated yet",
+                     priv->tag, state_nick);
+        skip = TRUE;
+    }
+
+    if (!priv->requesting_to_children) {
+        milter_debug("[%u] [leader][connection-check][%s][skip] "
+                     "not milter turn",
+                     priv->tag, state_nick);
+        skip = TRUE;
+    }
+
+    if (skip) {
+        if (state_nick)
+            g_free(state_nick);
+        return TRUE;
+    }
+
+    milter_debug("[%u] [leader][connection-check][%s][start][%g]",
+                 priv->tag, state_nick, elapsed);
+    if (priv->processing) {
+        g_signal_emit(leader, signals[CONNECTION_CHECK], 0, &connected);
+    } else {
+        connected = FALSE;
+    }
+    milter_debug("[%u] [leader][connection-check][%s][end][%s][%g]",
+                 priv->tag, state_nick,
+                 connected ? "connected" : "disconnected",
+                 elapsed);
+
+    if (!connected) {
+        MilterStatus fallback_status;
+
+        milter_statistics("[session][disconnected][%g](%u)", elapsed, priv->tag);
+
+        fallback_status =
+            milter_manager_configuration_get_fallback_status(
+                priv->configuration);
+        if (milter_need_debug_log()) {
+            gchar *fallback_status_nick;
+
+            fallback_status_nick =
+                milter_utils_get_enum_nick_name(MILTER_TYPE_STATUS,
+                                                fallback_status);
+            milter_debug("[%u] [leader][connection-check][%s][reply][%s]",
+                         priv->tag, state_nick, fallback_status_nick);
+            g_free(fallback_status_nick);
+        }
+        reply(leader, fallback_status);
+        if (priv->children) {
+            milter_manager_children_abort(priv->children);
+            milter_manager_children_quit(priv->children);
+        }
+    }
+
+    if (state_nick)
+        g_free(state_nick);
+
+    return connected;
 }
 
 static void
@@ -122,6 +255,8 @@ dispose (GObject *object)
     leader = MILTER_MANAGER_LEADER(object);
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(object);
 
+    milter_debug("[%u] [leader][dispose]", priv->tag);
+
     if (priv->configuration) {
         g_object_unref(priv->configuration);
         priv->configuration = NULL;
@@ -139,6 +274,7 @@ dispose (GObject *object)
     }
     milter_manager_leader_set_launcher_channel(leader, NULL, NULL);
 
+
     G_OBJECT_CLASS(milter_manager_leader_parent_class)->dispose(object);
 }
 
@@ -152,19 +288,22 @@ set_property (GObject      *object,
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(object);
     switch (prop_id) {
-      case PROP_CONFIGURATION:
+    case PROP_CONFIGURATION:
         if (priv->configuration)
             g_object_unref(priv->configuration);
         priv->configuration = g_value_get_object(value);
         if (priv->configuration)
             g_object_ref(priv->configuration);
         break;
-      case PROP_CLIENT_CONTEXT:
+    case PROP_CLIENT_CONTEXT:
+        priv->tag = 0;
         if (priv->client_context)
             g_object_unref(priv->client_context);
         priv->client_context = g_value_get_object(value);
-        if (priv->client_context)
+        if (priv->client_context) {
             g_object_ref(priv->client_context);
+            priv->tag = milter_agent_get_tag(MILTER_AGENT(priv->client_context));
+        }
         break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
@@ -188,12 +327,22 @@ get_property (GObject    *object,
       case PROP_CLIENT_CONTEXT:
         g_value_set_object(value, priv->client_context);
         break;
+      case PROP_CHILDREN:
+        g_value_set_object(value, priv->children);
+        break;
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
     }
 }
 
+static gboolean
+connection_check_default (MilterManagerLeader *leader)
+{
+    gboolean connecting = TRUE;
+    return connecting;
+}
+
 GQuark
 milter_manager_leader_error_quark (void)
 {
@@ -210,6 +359,32 @@ milter_manager_leader_new (MilterManagerConfiguration *configuration,
                         NULL);
 }
 
+static gboolean
+boolean_handled_accumulator (GSignalInvocationHint *ihint,
+                             GValue *return_accu,
+                             const GValue *handler_return,
+                             gpointer data)
+{
+    gboolean continue_emission;
+    gboolean signal_handled;
+
+    signal_handled = g_value_get_boolean(handler_return);
+    g_value_set_boolean(return_accu, signal_handled);
+    continue_emission = signal_handled;
+
+    return continue_emission;
+}
+
+static void
+finished (MilterFinishedEmittable *emittable)
+{
+    MilterManagerLeaderPrivate *priv;
+
+    priv = MILTER_MANAGER_LEADER_GET_PRIVATE(emittable);
+    priv->processing = FALSE;
+    priv->requesting_to_children = FALSE;
+}
+
 static const gchar *
 state_to_response_signal_name (MilterManagerLeaderState state)
 {
@@ -274,60 +449,58 @@ next_state (MilterManagerLeader *leader,
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
 
     switch (state) {
-      case MILTER_MANAGER_LEADER_STATE_NEGOTIATE:
+    case MILTER_MANAGER_LEADER_STATE_NEGOTIATE:
         next_state = MILTER_MANAGER_LEADER_STATE_NEGOTIATE_REPLIED;
         break;
-      case MILTER_MANAGER_LEADER_STATE_CONNECT:
+    case MILTER_MANAGER_LEADER_STATE_CONNECT:
         next_state = MILTER_MANAGER_LEADER_STATE_CONNECT_REPLIED;
         break;
-      case MILTER_MANAGER_LEADER_STATE_HELO:
+    case MILTER_MANAGER_LEADER_STATE_HELO:
         next_state = MILTER_MANAGER_LEADER_STATE_HELO_REPLIED;
         break;
-      case MILTER_MANAGER_LEADER_STATE_ENVELOPE_FROM:
+    case MILTER_MANAGER_LEADER_STATE_ENVELOPE_FROM:
         next_state = MILTER_MANAGER_LEADER_STATE_ENVELOPE_FROM_REPLIED;
         break;
-      case MILTER_MANAGER_LEADER_STATE_ENVELOPE_RECIPIENT:
+    case MILTER_MANAGER_LEADER_STATE_ENVELOPE_RECIPIENT:
         next_state = MILTER_MANAGER_LEADER_STATE_ENVELOPE_RECIPIENT_REPLIED;
         break;
-      case MILTER_MANAGER_LEADER_STATE_DATA:
+    case MILTER_MANAGER_LEADER_STATE_DATA:
         next_state = MILTER_MANAGER_LEADER_STATE_DATA_REPLIED;
         break;
-      case MILTER_MANAGER_LEADER_STATE_UNKNOWN:
+    case MILTER_MANAGER_LEADER_STATE_UNKNOWN:
         next_state = MILTER_MANAGER_LEADER_STATE_UNKNOWN_REPLIED;
         break;
-      case MILTER_MANAGER_LEADER_STATE_HEADER:
+    case MILTER_MANAGER_LEADER_STATE_HEADER:
         next_state = MILTER_MANAGER_LEADER_STATE_HEADER_REPLIED;
         break;
-      case MILTER_MANAGER_LEADER_STATE_END_OF_HEADER:
+    case MILTER_MANAGER_LEADER_STATE_END_OF_HEADER:
         next_state = MILTER_MANAGER_LEADER_STATE_END_OF_HEADER_REPLIED;
         break;
-      case MILTER_MANAGER_LEADER_STATE_BODY:
-      case MILTER_MANAGER_LEADER_STATE_BODY_REPLIED:
+    case MILTER_MANAGER_LEADER_STATE_BODY:
+    case MILTER_MANAGER_LEADER_STATE_BODY_REPLIED:
         if (!priv->sent_end_of_message)
             next_state = MILTER_MANAGER_LEADER_STATE_BODY_REPLIED;
         else
             next_state = MILTER_MANAGER_LEADER_STATE_END_OF_MESSAGE;
         break;
-      case MILTER_MANAGER_LEADER_STATE_END_OF_MESSAGE:
+    case MILTER_MANAGER_LEADER_STATE_END_OF_MESSAGE:
         next_state = MILTER_MANAGER_LEADER_STATE_END_OF_MESSAGE_REPLIED;
         break;
-      case MILTER_MANAGER_LEADER_STATE_QUIT:
+    case MILTER_MANAGER_LEADER_STATE_QUIT:
         next_state = MILTER_MANAGER_LEADER_STATE_QUIT_REPLIED;
         break;
-      case MILTER_MANAGER_LEADER_STATE_ABORT:
+    case MILTER_MANAGER_LEADER_STATE_ABORT:
         next_state = MILTER_MANAGER_LEADER_STATE_ABORT_REPLIED;
         break;
-      default:
-        {
+    default:
+        if (milter_need_error_log()) {
             gchar *inspected_state;
-            guint tag;
 
-            tag = milter_agent_get_tag(MILTER_AGENT(priv->client_context));
             inspected_state =
                 milter_utils_inspect_enum(MILTER_TYPE_MANAGER_LEADER_STATE,
                                           state);
             milter_error("[%u] [leader][error][invalid-state] %s",
-                         tag, inspected_state);
+                         priv->tag, inspected_state);
             g_free(inspected_state);
         }
         next_state = MILTER_MANAGER_LEADER_STATE_INVALID;
@@ -346,6 +519,8 @@ cb_negotiate_reply (MilterServerContext *context, MilterOption *option,
     MilterManagerLeaderPrivate *priv;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
+    priv->processing = TRUE;
+    priv->requesting_to_children = FALSE;
 
     g_signal_emit_by_name(priv->client_context, "negotiate-response",
                           option, macros_requests, MILTER_STATUS_CONTINUE);
@@ -357,6 +532,7 @@ reply (MilterManagerLeader *leader, MilterStatus status)
     MilterManagerLeaderPrivate *priv;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
+    priv->requesting_to_children = FALSE;
     if (priv->state == MILTER_MANAGER_LEADER_STATE_NEGOTIATE) {
         /* FIXME: should pass option and macros requests. */
         g_signal_emit_by_name(priv->client_context, "negotiate-response",
@@ -369,10 +545,8 @@ reply (MilterManagerLeader *leader, MilterStatus status)
             g_signal_emit_by_name(priv->client_context, signal_name, status);
         } else {
             GError *error;
-            guint tag;
             gchar *state_name, *status_name;
 
-            tag = milter_agent_get_tag(MILTER_AGENT(priv->client_context));
             state_name =
                 milter_utils_get_enum_nick_name(MILTER_TYPE_MANAGER_LEADER_STATE,
                                                 priv->state);
@@ -384,7 +558,7 @@ reply (MilterManagerLeader *leader, MilterStatus status)
                                 state_name, status_name);
             g_free(state_name);
             g_free(status_name);
-            milter_error("[%u] [leader][error] %s", tag, error->message);
+            milter_error("[%u] [leader][error] %s", priv->tag, error->message);
             milter_error_emittable_emit(MILTER_ERROR_EMITTABLE(leader), error);
             g_error_free(error);
             return;
@@ -416,7 +590,8 @@ cb_reply_code (MilterReplySignals *_reply,
                                     code, extended_code, message,
                                     &error);
     if (error) {
-        milter_error("[leader][error][reply-code] %s", error->message);
+        milter_error("[%u] [leader][error][reply-code] %s",
+                     priv->tag, error->message);
         milter_error_emittable_emit(MILTER_ERROR_EMITTABLE(leader),
                                     error);
         g_error_free(error);
@@ -470,14 +645,13 @@ cb_add_header (MilterReplySignals *_reply,
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
     milter_client_context_add_header(priv->client_context, name, value, &error);
     if (error) {
-        milter_error("[leader][error][add-header] %s", error->message);
+        milter_error("[%u] [leader][error][add-header] %s",
+                     priv->tag, error->message);
         milter_error_emittable_emit(MILTER_ERROR_EMITTABLE(leader), error);
         g_error_free(error);
     } else {
-        guint tag;
-        tag = milter_agent_get_tag(MILTER_AGENT(priv->client_context));
-        milter_statistics("[session][header][add](%u): <%s>=<%s>",
-                          tag, name, value);
+        milter_statistics(
+            "[session][header][add](%u): <%s>=<%s>", priv->tag, name, value);
     }
 }
 
@@ -494,14 +668,13 @@ cb_insert_header (MilterReplySignals *_reply,
     milter_client_context_insert_header(priv->client_context,
                                         index, name, value, &error);
     if (error) {
-        milter_error("[leader][error][insert-header] %s", error->message);
+        milter_error("[%u] [leader][error][insert-header] %s",
+                     priv->tag, error->message);
         milter_error_emittable_emit(MILTER_ERROR_EMITTABLE(leader), error);
         g_error_free(error);
     } else {
-        guint tag;
-        tag = milter_agent_get_tag(MILTER_AGENT(priv->client_context));
-        milter_statistics("[session][header][add](%u): <%s>=<%s>",
-                          tag, name, value);
+        milter_statistics(
+            "[session][header][add](%u): <%s>=<%s>", priv->tag, name, value);
     }
 }
 
@@ -518,10 +691,8 @@ cb_change_header (MilterReplySignals *_reply,
                                         name, index, value);
 
     if (value) {
-        guint tag;
-        tag = milter_agent_get_tag(MILTER_AGENT(priv->client_context));
-        milter_statistics("[session][header][add](%u): <%s>=<%s>",
-                          tag, name, value);
+        milter_statistics(
+            "[session][header][add](%u): <%s>=<%s>", priv->tag, name, value);
     }
 }
 
@@ -614,9 +785,11 @@ cb_connection_failure (MilterReplySignals *_reply, gpointer user_data)
     MilterManagerLeader *leader = user_data;
     MilterManagerLeaderPrivate *priv;
 
+    if (!milter_need_debug_log())
+        return;
+
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
-    milter_debug("[%u] [leader][unsupported][connection-failure]",
-                 milter_agent_get_tag(MILTER_AGENT(priv->client_context)));
+    milter_debug("[%u] [leader][unsupported][connection-failure]", priv->tag);
 }
 
 static void
@@ -625,9 +798,11 @@ cb_shutdown (MilterReplySignals *_reply, gpointer user_data)
     MilterManagerLeader *leader = user_data;
     MilterManagerLeaderPrivate *priv;
 
+    if (!milter_need_debug_log())
+        return;
+
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
-    milter_debug("[%u] [leader][unsupported][shutdown]",
-                 milter_agent_get_tag(MILTER_AGENT(priv->client_context)));
+    milter_debug("[%u] [leader][unsupported][shutdown]", priv->tag);
 }
 
 static void
@@ -642,17 +817,18 @@ cb_abort (MilterReplySignals *_reply, gpointer user_data)
 {
     MilterManagerLeader *leader = user_data;
     MilterManagerLeaderPrivate *priv;
-    gchar *state_name;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
 
-    state_name =
-        milter_utils_get_enum_nick_name(MILTER_TYPE_MANAGER_LEADER_STATE,
-                                        priv->state);
-    milter_debug("[%u] [leader][abort][%s]",
-                 milter_agent_get_tag(MILTER_AGENT(priv->client_context)),
-                 state_name);
-    g_free(state_name);
+    if (milter_need_debug_log()) {
+        gchar *state_name;
+
+        state_name =
+            milter_utils_get_enum_nick_name(MILTER_TYPE_MANAGER_LEADER_STATE,
+                                            priv->state);
+        milter_debug("[%u] [leader][abort][%s]", priv->tag, state_name);
+        g_free(state_name);
+    }
     milter_agent_shutdown(MILTER_AGENT(priv->client_context));
     milter_finished_emittable_emit(MILTER_FINISHED_EMITTABLE(leader));
 }
@@ -662,27 +838,19 @@ cb_error (MilterErrorEmittable *emittable, GError *error, gpointer user_data)
 {
     MilterManagerLeader *leader = user_data;
     MilterManagerLeaderPrivate *priv;
+    MilterStatus fallback_status;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
-    milter_error("[%u] [leader][error] %s",
-                 milter_agent_get_tag(MILTER_AGENT(priv->client_context)),
-                 error->message);
+    milter_error("[%u] [leader][error] %s", priv->tag, error->message);
     milter_error_emittable_emit(MILTER_ERROR_EMITTABLE(leader),
                                 error);
 
     if (error->domain != MILTER_MANAGER_CHILDREN_ERROR)
         return;
 
-    /* If all milters do not response on negotiation, send "reject" */
-    if (error->code == MILTER_MANAGER_CHILDREN_ERROR_NO_NEGOTIATION_RESPONSE) {
-        reply(leader, MILTER_STATUS_REJECT);
-    } else {
-        MilterStatus fallback_status;
-
-        fallback_status =
-            milter_manager_configuration_get_fallback_status(priv->configuration);
-        reply(leader, fallback_status);
-    }
+    fallback_status =
+        milter_manager_configuration_get_fallback_status(priv->configuration);
+    reply(leader, fallback_status);
 }
 
 static void
@@ -768,30 +936,34 @@ milter_manager_leader_negotiate (MilterManagerLeader *leader,
                                  MilterMacrosRequests *macros_requests)
 {
     MilterManagerLeaderPrivate *priv;
-    guint tag;
+    MilterStatus fallback_status;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
     priv->state = MILTER_MANAGER_LEADER_STATE_NEGOTIATE;
 
-    priv->children =
-        milter_manager_configuration_create_children(priv->configuration);
-
+    priv->children = milter_manager_children_new(priv->configuration);
+    milter_manager_configuration_setup_children(priv->configuration,
+                                                priv->children,
+                                                priv->client_context);
+    fallback_status =
+        milter_manager_configuration_get_fallback_status(priv->configuration);
     if (!priv->children)
-        return MILTER_STATUS_NOT_CHANGE;
+        return fallback_status;
 
-    tag = milter_agent_get_tag(MILTER_AGENT(priv->client_context));
-    milter_manager_children_set_tag(priv->children, tag);
+    milter_manager_children_set_tag(priv->children, priv->tag);
     setup_children_signals(leader, priv->children);
     milter_manager_children_set_launcher_channel(priv->children,
                                                  priv->launcher_read_channel,
                                                  priv->launcher_write_channel);
-    milter_debug("[%u] [leader][setup][children]", tag);
+    milter_debug("[%u] [leader][setup][children]", priv->tag);
 
     if (milter_manager_children_negotiate(priv->children, option,
-                                          macros_requests))
+                                          macros_requests)) {
+        priv->requesting_to_children = TRUE;
         return MILTER_STATUS_PROGRESS;
-    else
-        return MILTER_STATUS_NOT_CHANGE; /* FIXME: reject or accept */
+    } else {
+        return fallback_status;
+    }
 }
 
 MilterStatus
@@ -801,35 +973,45 @@ milter_manager_leader_connect (MilterManagerLeader *leader,
                                socklen_t            address_length)
 {
     MilterManagerLeaderPrivate *priv;
+    MilterStatus fallback_status;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
     priv->state = MILTER_MANAGER_LEADER_STATE_CONNECT;
 
+    fallback_status =
+        milter_manager_configuration_get_fallback_status(priv->configuration);
     if (!priv->children)
-        return MILTER_STATUS_NOT_CHANGE;
+        return fallback_status;
 
     if (milter_manager_children_connect(priv->children, host_name,
-                                        address, address_length))
+                                        address, address_length)) {
+        priv->requesting_to_children = TRUE;
         return MILTER_STATUS_PROGRESS;
-    else
-        return MILTER_STATUS_NOT_CHANGE; /* FIXME: reject or accept */
+    } else {
+        return fallback_status;
+    }
 }
 
 MilterStatus
 milter_manager_leader_helo (MilterManagerLeader *leader, const gchar *fqdn)
 {
     MilterManagerLeaderPrivate *priv;
+    MilterStatus fallback_status;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
     priv->state = MILTER_MANAGER_LEADER_STATE_HELO;
 
+    fallback_status =
+        milter_manager_configuration_get_fallback_status(priv->configuration);
     if (!priv->children)
-        return MILTER_STATUS_NOT_CHANGE;
+        return fallback_status;
 
-    if (milter_manager_children_helo(priv->children, fqdn))
+    if (milter_manager_children_helo(priv->children, fqdn)) {
+        priv->requesting_to_children = TRUE;
         return MILTER_STATUS_PROGRESS;
-    else
-        return MILTER_STATUS_NOT_CHANGE; /* FIXME: reject or accept */
+    } else {
+        return fallback_status;
+    }
 }
 
 MilterStatus
@@ -837,17 +1019,22 @@ milter_manager_leader_envelope_from (MilterManagerLeader *leader,
                                      const gchar *from)
 {
     MilterManagerLeaderPrivate *priv;
+    MilterStatus fallback_status;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
     priv->state = MILTER_MANAGER_LEADER_STATE_ENVELOPE_FROM;
 
+    fallback_status =
+        milter_manager_configuration_get_fallback_status(priv->configuration);
     if (!priv->children)
-        return MILTER_STATUS_NOT_CHANGE;
+        return fallback_status;
 
-    if (milter_manager_children_envelope_from(priv->children, from))
+    if (milter_manager_children_envelope_from(priv->children, from)) {
+        priv->requesting_to_children = TRUE;
         return MILTER_STATUS_PROGRESS;
-    else
-        return MILTER_STATUS_NOT_CHANGE; /* FIXME: reject or accept */
+    } else {
+        return fallback_status;
+    }
 }
 
 MilterStatus
@@ -855,34 +1042,44 @@ milter_manager_leader_envelope_recipient (MilterManagerLeader *leader,
                                           const gchar *recipient)
 {
     MilterManagerLeaderPrivate *priv;
+    MilterStatus fallback_status;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
     priv->state = MILTER_MANAGER_LEADER_STATE_ENVELOPE_RECIPIENT;
 
+    fallback_status =
+        milter_manager_configuration_get_fallback_status(priv->configuration);
     if (!priv->children)
-        return MILTER_STATUS_NOT_CHANGE;
+        return fallback_status;
 
-    if (milter_manager_children_envelope_recipient(priv->children, recipient))
+    if (milter_manager_children_envelope_recipient(priv->children, recipient)) {
+        priv->requesting_to_children = TRUE;
         return MILTER_STATUS_PROGRESS;
-    else
-        return MILTER_STATUS_NOT_CHANGE; /* FIXME: reject or accept */
+    } else {
+        return fallback_status;
+    }
 }
 
 MilterStatus
 milter_manager_leader_data (MilterManagerLeader *leader)
 {
     MilterManagerLeaderPrivate *priv;
+    MilterStatus fallback_status;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
     priv->state = MILTER_MANAGER_LEADER_STATE_DATA;
 
+    fallback_status =
+        milter_manager_configuration_get_fallback_status(priv->configuration);
     if (!priv->children)
-        return MILTER_STATUS_NOT_CHANGE;
+        return fallback_status;
 
-    if (milter_manager_children_data(priv->children))
+    if (milter_manager_children_data(priv->children)) {
+        priv->requesting_to_children = TRUE;
         return MILTER_STATUS_PROGRESS;
-    else
-        return MILTER_STATUS_NOT_CHANGE; /* FIXME: reject or accept */
+    } else {
+        return fallback_status;
+    }
 }
 
 MilterStatus
@@ -890,17 +1087,22 @@ milter_manager_leader_unknown (MilterManagerLeader *leader,
                                const gchar *command)
 {
     MilterManagerLeaderPrivate *priv;
+    MilterStatus fallback_status;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
     priv->state = MILTER_MANAGER_LEADER_STATE_UNKNOWN;
 
+    fallback_status =
+        milter_manager_configuration_get_fallback_status(priv->configuration);
     if (!priv->children)
-        return MILTER_STATUS_NOT_CHANGE;
+        return fallback_status;
 
-    if (milter_manager_children_unknown(priv->children, command))
+    if (milter_manager_children_unknown(priv->children, command)) {
+        priv->requesting_to_children = TRUE;
         return MILTER_STATUS_PROGRESS;
-    else
-        return MILTER_STATUS_NOT_CHANGE; /* FIXME: reject or accept */
+    } else {
+        return fallback_status;
+    }
 }
 
 MilterStatus
@@ -908,34 +1110,44 @@ milter_manager_leader_header (MilterManagerLeader *leader,
                               const gchar *name, const gchar *value)
 {
     MilterManagerLeaderPrivate *priv;
+    MilterStatus fallback_status;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
     priv->state = MILTER_MANAGER_LEADER_STATE_HEADER;
 
+    fallback_status =
+        milter_manager_configuration_get_fallback_status(priv->configuration);
     if (!priv->children)
-        return MILTER_STATUS_NOT_CHANGE;
+        return fallback_status;
 
-    if (milter_manager_children_header(priv->children, name, value))
+    if (milter_manager_children_header(priv->children, name, value)) {
+        priv->requesting_to_children = TRUE;
         return MILTER_STATUS_PROGRESS;
-    else
-        return MILTER_STATUS_NOT_CHANGE; /* FIXME: reject or accept */
+    } else {
+        return fallback_status;
+    }
 }
 
 MilterStatus
 milter_manager_leader_end_of_header (MilterManagerLeader *leader)
 {
     MilterManagerLeaderPrivate *priv;
+    MilterStatus fallback_status;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
     priv->state = MILTER_MANAGER_LEADER_STATE_END_OF_HEADER;
 
+    fallback_status =
+        milter_manager_configuration_get_fallback_status(priv->configuration);
     if (!priv->children)
-        return MILTER_STATUS_NOT_CHANGE;
+        return fallback_status;
 
-    if (milter_manager_children_end_of_header(priv->children))
+    if (milter_manager_children_end_of_header(priv->children)) {
+        priv->requesting_to_children = TRUE;
         return MILTER_STATUS_PROGRESS;
-    else
-        return MILTER_STATUS_NOT_CHANGE; /* FIXME: reject or accept */
+    } else {
+        return fallback_status;
+    }
 }
 
 MilterStatus
@@ -943,17 +1155,22 @@ milter_manager_leader_body (MilterManagerLeader *leader,
                             const gchar *chunk, gsize size)
 {
     MilterManagerLeaderPrivate *priv;
+    MilterStatus fallback_status;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
     priv->state = MILTER_MANAGER_LEADER_STATE_BODY;
 
+    fallback_status =
+        milter_manager_configuration_get_fallback_status(priv->configuration);
     if (!priv->children)
-        return MILTER_STATUS_NOT_CHANGE;
+        return fallback_status;
 
-    if (milter_manager_children_body(priv->children, chunk, size))
+    if (milter_manager_children_body(priv->children, chunk, size)) {
+        priv->requesting_to_children = TRUE;
         return MILTER_STATUS_PROGRESS;
-    else
-        return MILTER_STATUS_NOT_CHANGE; /* FIXME: reject or accept */
+    } else {
+        return fallback_status;
+    }
 }
 
 static gboolean
@@ -986,6 +1203,7 @@ milter_manager_leader_end_of_message (MilterManagerLeader *leader,
                                       gsize                size)
 {
     MilterManagerLeaderPrivate *priv;
+    MilterStatus fallback_status;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
     if (is_replied_state(priv->state)) {
@@ -994,25 +1212,34 @@ milter_manager_leader_end_of_message (MilterManagerLeader *leader,
         priv->sent_end_of_message = TRUE;
     }
 
+    fallback_status =
+        milter_manager_configuration_get_fallback_status(priv->configuration);
     if (!priv->children)
-        return MILTER_STATUS_NOT_CHANGE;
+        return fallback_status;
 
-    if (milter_manager_children_end_of_message(priv->children, chunk, size))
+    if (milter_manager_children_end_of_message(priv->children, chunk, size)) {
+        priv->requesting_to_children = TRUE;
         return MILTER_STATUS_PROGRESS;
-    else
-        return MILTER_STATUS_NOT_CHANGE; /* FIXME: reject or accept */
+    } else {
+        return fallback_status;
+    }
 }
 
 MilterStatus
 milter_manager_leader_quit (MilterManagerLeader *leader)
 {
     MilterManagerLeaderPrivate *priv;
+    MilterStatus fallback_status;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
     priv->state = MILTER_MANAGER_LEADER_STATE_QUIT;
 
-    if (priv->children)
-        milter_manager_children_quit(priv->children);
+    fallback_status =
+        milter_manager_configuration_get_fallback_status(priv->configuration);
+    if (!priv->children)
+        return fallback_status;
+
+    milter_manager_children_quit(priv->children);
     return MILTER_STATUS_DEFAULT;
 }
 
@@ -1020,12 +1247,17 @@ MilterStatus
 milter_manager_leader_abort (MilterManagerLeader *leader)
 {
     MilterManagerLeaderPrivate *priv;
+    MilterStatus fallback_status;
 
     priv = MILTER_MANAGER_LEADER_GET_PRIVATE(leader);
     priv->state = MILTER_MANAGER_LEADER_STATE_ABORT;
 
-    if (priv->children)
-        milter_manager_children_abort(priv->children);
+    fallback_status =
+        milter_manager_configuration_get_fallback_status(priv->configuration);
+    if (!priv->children)
+        return fallback_status;
+
+    milter_manager_children_abort(priv->children);
     return MILTER_STATUS_DEFAULT;
 }
 
@@ -1079,6 +1311,12 @@ milter_manager_leader_set_launcher_channel (MilterManagerLeader *leader,
         g_io_channel_ref(priv->launcher_read_channel);
 }
 
+MilterManagerChildren *
+milter_manager_leader_get_children (MilterManagerLeader *leader)
+{
+    return MILTER_MANAGER_LEADER_GET_PRIVATE(leader)->children;
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
