@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -171,6 +171,13 @@ milter_manager_init (int *argc, char ***argv)
     milter_server_init();
     milter_manager_log_handler_id = MILTER_GLIB_LOG_DELEGATE("milter-manager");
 
+    if (milter_get_log_level() == MILTER_LOG_LEVEL_DEFAULT) {
+        milter_set_log_level(MILTER_LOG_LEVEL_CRITICAL |
+                             MILTER_LOG_LEVEL_ERROR |
+                             MILTER_LOG_LEVEL_WARNING |
+                             MILTER_LOG_LEVEL_STATISTICS);
+    }
+
     option_context = g_option_context_new(NULL);
     g_option_context_add_main_entries(option_context, option_entries, NULL);
     main_group = g_option_context_get_main_group(option_context);
@@ -196,7 +203,7 @@ milter_manager_init (int *argc, char ***argv)
     }
 
     if (option_verbose)
-        g_setenv("MILTER_LOG_LEVEL", "all", TRUE);
+        milter_set_log_level(MILTER_LOG_LEVEL_ALL);
 
     _milter_manager_configuration_init();
     g_option_context_free(option_context);
@@ -359,6 +366,7 @@ start_process_launcher (GIOChannel *read_channel, GIOChannel *write_channel)
     MilterManagerProcessLauncher *launcher;
     MilterReader *reader;
     MilterWriter *writer;
+    gchar *error_message = NULL;
 
     reader = milter_reader_io_channel_new(read_channel);
     g_io_channel_unref(read_channel);
@@ -371,7 +379,13 @@ start_process_launcher (GIOChannel *read_channel, GIOChannel *write_channel)
     g_object_unref(reader);
     g_object_unref(writer);
 
-    milter_agent_start(MILTER_AGENT(launcher));
+    milter_agent_start(MILTER_AGENT(launcher), NULL);
+
+    if (!milter_utils_detach_io(&error_message)) {
+        milter_error("[process-launcher][error] failed to detach IO: %s",
+                     error_message);
+        g_free(error_message);
+    }
 
     milter_manager_process_launcher_run(launcher);
 
@@ -456,6 +470,7 @@ update_max_file_descriptors (MilterManager *manager)
 {
     MilterManagerConfiguration *configuration;
     guint max_file_descriptors;
+    struct rlimit original_rlimit;
     struct rlimit rlimit;
 
     configuration = milter_manager_get_configuration(manager);
@@ -465,17 +480,25 @@ update_max_file_descriptors (MilterManager *manager)
     if (max_file_descriptors == 0)
         return;
 
-    if (getrlimit(RLIMIT_NOFILE, &rlimit) != 0) {
+    if (getrlimit(RLIMIT_NOFILE, &original_rlimit) != 0) {
         milter_manager_error("failed to get limit for RLIMIT_NOFILE: %s",
                              g_strerror(errno));
         return;
     }
 
-    rlimit.rlim_cur = MAX(max_file_descriptors, rlimit.rlim_max);
+    memcpy(&rlimit, &original_rlimit, sizeof(original_rlimit));
+    rlimit.rlim_cur = max_file_descriptors;
+    rlimit.rlim_max = max_file_descriptors;
     if (setrlimit(RLIMIT_NOFILE, &rlimit) != 0) {
         milter_manager_error("failed to set limit for RLIMIT_NOFILE: "
-                             "%" G_GINT64_FORMAT ": %s",
-                             (gint64)rlimit.rlim_max, g_strerror(errno));
+                             "%u "
+                             "original soft limit: %" G_GINT64_FORMAT " "
+                             "original hard limit: %" G_GINT64_FORMAT " "
+                             ": %s",
+                             max_file_descriptors,
+                             (gint64)original_rlimit.rlim_cur,
+                             (gint64)original_rlimit.rlim_max,
+                             g_strerror(errno));
         return;
     }
 }
@@ -499,8 +522,6 @@ apply_command_line_options (MilterManagerConfiguration *config)
             config, option_socket_group_name);
     if (option_daemon)
         milter_manager_configuration_set_daemon(config, TRUE);
-    if (getuid() != 0)
-        milter_manager_configuration_set_privilege_mode(config, FALSE);
 }
 
 static void
@@ -527,7 +548,7 @@ append_custom_configuration_directory (MilterManagerConfiguration *config)
     if (!g_file_test(custom_config_directory, G_FILE_TEST_EXISTS)) {
         if (g_mkdir(custom_config_directory, 0700) == -1) {
             milter_manager_error("failed to create custom "
-                                 "configuration directory: %s: %s",
+                                 "configuration directory: <%s>: %s",
                                  custom_config_directory,
                                  g_strerror(errno));
             return;
@@ -630,6 +651,8 @@ milter_manager_main (void)
                                                        option_config_dir);
     milter_manager_configuration_reload(config);
     apply_command_line_options(config);
+    if (getuid() != 0)
+        milter_manager_configuration_set_privilege_mode(config, FALSE);
 
     manager = milter_manager_new(config);
     g_object_unref(config);
