@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -26,6 +26,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <pwd.h>
+#include <grp.h>
 #include <errno.h>
 
 #include "milter-manager-process-launcher.h"
@@ -101,43 +102,6 @@ milter_manager_process_launcher_class_init (MilterManagerProcessLauncherClass *k
                              sizeof(MilterManagerProcessLauncherPrivate));
 }
 
-static gboolean
-set_user (const gchar *user_name, GError **error)
-{
-    struct passwd *password;
-
-    errno = 0;
-    password = getpwnam(user_name);
-    if (!password) {
-        g_set_error(error,
-                    MILTER_MANAGER_CHILD_ERROR,
-                    MILTER_MANAGER_CHILD_ERROR_INVALID_USER_NAME,
-                    "No password entry: <%s>%s%s",
-                    user_name,
-                    errno == 0 ? "" : ": ",
-                    errno == 0 ? "" : g_strerror(errno));
-        return FALSE;
-    }
-
-    if (setregid(-1, password->pw_gid) == -1) {
-        g_set_error(error,
-                    MILTER_MANAGER_CHILD_ERROR,
-                    MILTER_MANAGER_CHILD_ERROR_NO_PRIVILEGE_MODE,
-                    "failed to change GID: %s", g_strerror(errno));
-        return FALSE;
-    }
-
-    if (setreuid(password->pw_uid, -1) == -1) {
-        g_set_error(error,
-                    MILTER_MANAGER_CHILD_ERROR,
-                    MILTER_MANAGER_CHILD_ERROR_NO_PRIVILEGE_MODE,
-                    "failed to change UID: %s", g_strerror(errno));
-        return FALSE;
-    }
-
-    return TRUE;
-}
-
 static ProcessData *
 process_data_new (GPid pid, guint watch_id,
                   const gchar *command_line, const gchar *user_name)
@@ -230,6 +194,32 @@ is_already_launched (MilterManagerProcessLauncher *launcher,
     return TRUE;
 }
 
+static void
+setup_child (gpointer user_data)
+{
+    struct passwd *password = user_data;
+
+    if (!password)
+        return;
+
+    if (setgid(password->pw_gid) == -1) {
+        milter_error("failed to change GID: %s", g_strerror(errno));
+        return;
+    }
+
+    if (initgroups(password->pw_name, password->pw_gid) == -1) {
+        milter_error("failed to initialize group access: %s", g_strerror(errno));
+        return;
+    }
+
+    if (setuid(password->pw_uid) == -1) {
+        milter_error("failed to change UID: %s", g_strerror(errno));
+        return;
+    }
+}
+
+#define PASSWORD_DATA_BUFFER_SIZE 512
+
 static gboolean
 launch (MilterManagerProcessLauncher *launcher,
         const gchar *command_line, const gchar *user_name,
@@ -243,12 +233,31 @@ launch (MilterManagerProcessLauncher *launcher,
     GPid pid;
     guint watch_id;
     MilterManagerProcessLauncherPrivate *priv;
+    struct passwd password;
+    struct passwd *password_p = NULL;
+    gchar password_data_buffer[PASSWORD_DATA_BUFFER_SIZE];
+
+    milter_debug("[launcher][launch] <%s>@<%s>", command_line, user_name);
 
     if (is_already_launched(launcher, command_line, error))
         return FALSE;
 
-    if (user_name && !set_user(user_name, error))
-        return FALSE;
+    if (user_name) {
+        int result;
+        result = getpwnam_r(user_name, &password,
+                            password_data_buffer, PASSWORD_DATA_BUFFER_SIZE,
+                            &password_p);
+        if (!password_p) {
+            g_set_error(error,
+                        MILTER_MANAGER_CHILD_ERROR,
+                        MILTER_MANAGER_CHILD_ERROR_INVALID_USER_NAME,
+                        "No password entry: <%s>%s%s",
+                        user_name,
+                        result == 0 ? "" : ": ",
+                        result == 0 ? "" : g_strerror(result));
+            return FALSE;
+        }
+    }
 
     success = g_shell_parse_argv(command_line,
                                  &argc, &argv,
@@ -273,8 +282,8 @@ launch (MilterManagerProcessLauncher *launcher,
                             argv,
                             NULL,
                             flags,
-                            NULL,
-                            NULL,
+                            setup_child,
+                            password_p,
                             &pid,
                             &internal_error);
     g_strfreev(argv);
