@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -42,9 +42,16 @@ struct _MilterManagerPrivate
     MilterManagerConfiguration *configuration;
     MilterSyslogLogger *logger;
     GList *leaders;
+    GList *next_connection_checked_leader;
+    gboolean connection_checking;
 
     GIOChannel *launcher_read_channel;
     GIOChannel *launcher_write_channel;
+
+    guint periodical_connection_checker_id;
+    guint current_periodical_connection_check_interval;
+
+    GList *finished_leaders;
 };
 
 enum
@@ -76,10 +83,8 @@ static guint  get_suspend_time_on_unacceptable
 static void   set_suspend_time_on_unacceptable
                                           (MilterClient *client,
                                            guint         suspend_time);
-static guint  get_max_connections
-                                          (MilterClient *client);
-static void   set_max_connections
-                                          (MilterClient *client,
+static guint  get_max_connections         (MilterClient *client);
+static void   set_max_connections         (MilterClient *client,
                                            guint         max_connections);
 static const gchar *get_effective_user    (MilterClient *client);
 static void   set_effective_user          (MilterClient *client,
@@ -87,6 +92,12 @@ static void   set_effective_user          (MilterClient *client,
 static const gchar *get_effective_group   (MilterClient *client);
 static void   set_effective_group         (MilterClient *client,
                                            const gchar  *effective_group);
+static guint  get_maintenance_interval    (MilterClient *client);
+static void   set_maintenance_interval    (MilterClient *client,
+                                           guint         n_sessions);
+static void   maintain                    (MilterClient *client);
+static void   sessions_finished           (MilterClient *client,
+                                           guint         n_finished_sessions);
 static void   cb_leader_finished          (MilterFinishedEmittable *emittable,
                                            gpointer user_data);
 
@@ -117,6 +128,10 @@ milter_manager_class_init (MilterManagerClass *klass)
     client_class->set_effective_user = set_effective_user;
     client_class->get_effective_group = get_effective_group;
     client_class->set_effective_group = set_effective_group;
+    client_class->get_maintenance_interval = get_maintenance_interval;
+    client_class->set_maintenance_interval = set_maintenance_interval;
+    client_class->maintain = maintain;
+    client_class->sessions_finished = sessions_finished;
 
     spec = g_param_spec_object("configuration",
                                "Configuration",
@@ -138,10 +153,56 @@ milter_manager_init (MilterManager *manager)
 
     priv->configuration = NULL;
     priv->leaders = NULL;
+    priv->next_connection_checked_leader = NULL;
+    priv->connection_checking = FALSE;
     priv->logger = milter_syslog_logger_new("milter-manager");
 
     priv->launcher_read_channel = NULL;
     priv->launcher_write_channel = NULL;
+
+    priv->periodical_connection_checker_id = 0;
+    priv->current_periodical_connection_check_interval = 0;
+
+    priv->finished_leaders = NULL;
+}
+
+static void
+configuration_set_manager (MilterManagerConfiguration *configuration,
+                           MilterManager *manager)
+{
+    g_object_set_data(G_OBJECT(configuration), "manager", manager);
+}
+
+static void
+dispose_periodical_connection_checker (MilterManagerPrivate *priv)
+{
+    priv->current_periodical_connection_check_interval = 0;
+
+    if (priv->periodical_connection_checker_id == 0)
+        return;
+
+    g_source_remove(priv->periodical_connection_checker_id);
+    priv->periodical_connection_checker_id = 0;
+}
+
+static void
+dispose_finished_leaders (MilterManagerPrivate *priv)
+{
+    GList *node;
+    guint n_leaders = 0;
+
+    if (!priv->finished_leaders)
+        return;
+
+    for (node = priv->finished_leaders; node; node = g_list_next(node)) {
+        MilterManagerLeader *leader = node->data;
+        g_object_unref(leader);
+        n_leaders++;
+    }
+    g_list_free(priv->finished_leaders);
+    priv->finished_leaders = NULL;
+
+    milter_debug("[manager][dispose][leaders] %u", n_leaders);
 }
 
 static void
@@ -151,21 +212,26 @@ dispose (GObject *object)
 
     priv = MILTER_MANAGER_GET_PRIVATE(object);
 
+    dispose_periodical_connection_checker(priv);
+    dispose_finished_leaders(priv);
+
     if (priv->configuration) {
+        configuration_set_manager(priv->configuration, NULL);
         g_object_unref(priv->configuration);
         priv->configuration = NULL;
     }
 
     if (priv->leaders) {
-        g_list_foreach(priv->leaders, (GFunc)g_object_unref, NULL);
         g_list_free(priv->leaders);
         priv->leaders = NULL;
     }
+    priv->next_connection_checked_leader = NULL;
 
     if (priv->logger) {
         g_object_unref(priv->logger);
         priv->logger = NULL;
     }
+
     milter_manager_set_launcher_channel(MILTER_MANAGER(object), NULL, NULL);
 
     G_OBJECT_CLASS(milter_manager_parent_class)->dispose(object);
@@ -177,18 +243,24 @@ set_property (GObject      *object,
               const GValue *value,
               GParamSpec   *pspec)
 {
+    MilterManager *manager;
     MilterManagerPrivate *priv;
 
-    priv = MILTER_MANAGER_GET_PRIVATE(object);
+    manager = MILTER_MANAGER(object);
+    priv = MILTER_MANAGER_GET_PRIVATE(manager);
     switch (prop_id) {
-      case PROP_CONFIGURATION:
-        if (priv->configuration)
+    case PROP_CONFIGURATION:
+        if (priv->configuration) {
+            configuration_set_manager(priv->configuration, NULL);
             g_object_unref(priv->configuration);
+        }
         priv->configuration = g_value_get_object(value);
-        if (priv->configuration)
+        if (priv->configuration) {
             g_object_ref(priv->configuration);
+            configuration_set_manager(priv->configuration, manager);
+        }
         break;
-      default:
+    default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
     }
@@ -221,20 +293,85 @@ milter_manager_new (MilterManagerConfiguration *configuration)
                         NULL);
 }
 
+static gboolean
+connection_check (gpointer data)
+{
+    MilterManager *manager = data;
+    MilterManagerPrivate *priv;
+    GList *node, *next_node;
+    guint i;
+    guint n_leaders_per_connection_check = 30; /* FIXME: make customizable */
+
+    priv = MILTER_MANAGER_GET_PRIVATE(manager);
+
+    priv->connection_checking = TRUE;
+
+    if (!priv->next_connection_checked_leader)
+        priv->next_connection_checked_leader = priv->leaders;
+    node = priv->next_connection_checked_leader;
+    i = 0;
+    while (node && i < n_leaders_per_connection_check) {
+        MilterManagerLeader *leader = node->data;
+        next_node = g_list_next(node);
+        if (!milter_manager_leader_check_connection(leader)) {
+            priv->leaders = g_list_delete_link(priv->leaders, node);
+        }
+        node = next_node;
+        i++;
+    }
+    priv->next_connection_checked_leader = node;
+
+    priv->connection_checking = FALSE;
+
+    return priv->leaders != NULL;
+}
+
+static void
+start_periodical_connection_checker (MilterManager *manager)
+{
+    MilterManagerPrivate *priv;
+    guint interval;
+
+    priv = MILTER_MANAGER_GET_PRIVATE(manager);
+
+    interval = milter_manager_configuration_get_connection_check_interval(priv->configuration);
+
+    if (interval > 0) {
+        if (priv->periodical_connection_checker_id == 0 ||
+            interval != priv->current_periodical_connection_check_interval) {
+            milter_debug("[manager][connection-check][start] <%u> -> <%u>: %s",
+                         priv->current_periodical_connection_check_interval,
+                         interval,
+                         priv->periodical_connection_checker_id == 0 ?
+                         "initial" : "update");
+            dispose_periodical_connection_checker(priv);
+            priv->current_periodical_connection_check_interval = interval;
+            priv->periodical_connection_checker_id =
+                g_timeout_add(interval * 1000, connection_check, manager);
+        }
+    } else {
+        dispose_periodical_connection_checker(priv);
+    }
+}
+
 static MilterStatus
 cb_client_negotiate (MilterClientContext *context, MilterOption *option,
                      MilterMacrosRequests *macros_requests, gpointer user_data)
 {
     MilterManagerLeader *leader = user_data;
-    gchar *inspected_option, *inspected_macros_requests;
 
-    inspected_option = milter_option_inspect(option);
-    inspected_macros_requests = milter_utils_inspect_object(G_OBJECT(macros_requests));
-    milter_debug("[%u] [manager][receive][negotiate] <%s>:<%s>",
-                 milter_agent_get_tag(MILTER_AGENT(context)),
-                 inspected_option, inspected_macros_requests);
-    g_free(inspected_option);
-    g_free(inspected_macros_requests);
+    if (milter_need_debug_log()) {
+        gchar *inspected_option, *inspected_macros_requests;
+
+        inspected_option = milter_option_inspect(option);
+        inspected_macros_requests =
+            milter_utils_inspect_object(G_OBJECT(macros_requests));
+        milter_debug("[%u] [manager][receive][negotiate] <%s>:<%s>",
+                     milter_agent_get_tag(MILTER_AGENT(context)),
+                     inspected_option, inspected_macros_requests);
+        g_free(inspected_option);
+        g_free(inspected_macros_requests);
+    }
 
     return milter_manager_leader_negotiate(leader, option, macros_requests);
 }
@@ -245,13 +382,16 @@ cb_client_connect (MilterClientContext *context, const gchar *host_name,
                    gpointer user_data)
 {
     MilterManagerLeader *leader = user_data;
-    gchar *spec;
 
-    spec = milter_connection_address_to_spec(address);
-    milter_debug("[%u] [manager][receive][connect] <%s>:<%s>",
-                 milter_agent_get_tag(MILTER_AGENT(context)),
-                 host_name, spec);
-    g_free(spec);
+    if (milter_need_debug_log()) {
+        gchar *spec;
+
+        spec = milter_connection_address_to_spec(address);
+        milter_debug("[%u] [manager][receive][connect] <%s>:<%s>",
+                     milter_agent_get_tag(MILTER_AGENT(context)),
+                     host_name, spec);
+        g_free(spec);
+    }
 
     return milter_manager_leader_connect(leader, host_name,
                                          address, address_length);
@@ -378,15 +518,18 @@ cb_client_abort (MilterClientContext *context, MilterClientContextState state,
                  gpointer user_data)
 {
     MilterManagerLeader *leader = user_data;
-    gchar *state_name;
 
-    state_name =
-        milter_utils_get_enum_nick_name(MILTER_TYPE_CLIENT_CONTEXT_STATE,
-                                        state);
-    milter_debug("[%u] [manager][receive][abort][%s]",
-                 milter_agent_get_tag(MILTER_AGENT(context)),
-                 state_name);
-    g_free(state_name);
+    if (milter_need_debug_log()) {
+        gchar *state_name;
+
+        state_name =
+            milter_utils_get_enum_nick_name(MILTER_TYPE_CLIENT_CONTEXT_STATE,
+                                            state);
+        milter_debug("[%u] [manager][receive][abort][%s]",
+                     milter_agent_get_tag(MILTER_AGENT(context)),
+                     state_name);
+        g_free(state_name);
+    }
 
     return milter_manager_leader_abort(leader);
 }
@@ -396,13 +539,16 @@ cb_client_define_macro (MilterClientContext *context, MilterCommand command,
                         GHashTable *macros, gpointer user_data)
 {
     MilterManagerLeader *leader = user_data;
-    gchar *inspected_macros;
 
-    inspected_macros = milter_utils_inspect_hash_string_string(macros);
-    milter_debug("[%u] [manager][receive][define-macro] <%s>",
-                 milter_agent_get_tag(MILTER_AGENT(context)),
-                 inspected_macros);
-    g_free(inspected_macros);
+    if (milter_need_debug_log()) {
+        gchar *inspected_macros;
+
+        inspected_macros = milter_utils_inspect_hash_string_string(macros);
+        milter_debug("[%u] [manager][receive][define-macro] <%s>",
+                     milter_agent_get_tag(MILTER_AGENT(context)),
+                     inspected_macros);
+        g_free(inspected_macros);
+    }
 
     milter_manager_leader_define_macro(leader, command, macros);
 }
@@ -422,50 +568,68 @@ static void
 cb_client_finished (MilterClientContext *context, gpointer user_data)
 {
     MilterManagerLeader *leader = user_data;
-    GTimer *timer;
-    gdouble elapsed;
-    MilterClientContextState state, last_state;
-    MilterStatus status;
-    gchar *state_name, *last_state_name;
-    gchar *status_name, *statistics_status_name;
-    guint tag;
 
-    timer = milter_client_context_get_private_data(context);
-    g_timer_stop(timer);
-    elapsed = g_timer_elapsed(timer, NULL);
+    if (milter_need_log(MILTER_LOG_LEVEL_DEBUG |
+                        MILTER_LOG_LEVEL_STATISTICS)) {
+        MilterAgent *agent;
+        guint tag;
+        gdouble elapsed;
+        MilterStatus status;
+        gchar *status_name, *statistics_status_name;
 
+        agent = MILTER_AGENT(context);
+        tag = milter_agent_get_tag(agent);
+        elapsed = milter_agent_get_elapsed(agent);
 
-    state = milter_client_context_get_state(context);
-    state_name =
-        milter_utils_get_enum_nick_name(MILTER_TYPE_CLIENT_CONTEXT_STATE,
-                                        state);
-    last_state = milter_client_context_get_last_state(context);
-    last_state_name =
-        milter_utils_get_enum_nick_name(MILTER_TYPE_CLIENT_CONTEXT_STATE,
-                                        last_state);
-    status = milter_client_context_get_status(context);
-    status_name = milter_utils_get_enum_nick_name(MILTER_TYPE_STATUS,
-                                                  status);
-    tag = milter_agent_get_tag(MILTER_AGENT(context));
-    milter_debug("[%u] [manager][finish][%s][%s][%g]",
-                 tag, state_name, status_name, elapsed);
-    if (MILTER_STATUS_IS_PASS(status)) {
-        statistics_status_name = "pass";
-    } else {
-        statistics_status_name = status_name;
+        status = milter_client_context_get_status(context);
+        status_name = milter_utils_get_enum_nick_name(MILTER_TYPE_STATUS,
+                                                      status);
+
+        if (milter_need_debug_log()) {
+            MilterClientContextState state;
+            gchar *state_name;
+            state = milter_client_context_get_state(context);
+            state_name = milter_utils_get_enum_nick_name(
+                MILTER_TYPE_CLIENT_CONTEXT_STATE, state);
+            milter_debug("[%u] [manager][finish][%s][%s][%g]",
+                         tag, state_name, status_name, elapsed);
+            g_free(state_name);
+        }
+
+        if (milter_need_statistics_log()) {
+            MilterClientContextState last_state;
+            gchar *last_state_name;
+
+            last_state = milter_client_context_get_last_state(context);
+            last_state_name =
+                milter_utils_get_enum_nick_name(
+                    MILTER_TYPE_CLIENT_CONTEXT_STATE, last_state);
+            if (MILTER_STATUS_IS_PASS(status)) {
+                statistics_status_name = "pass";
+            } else {
+                statistics_status_name = status_name;
+            }
+            milter_statistics("[session][end][%s][%s][%g](%u)",
+                              last_state_name, statistics_status_name,
+                              elapsed, tag);
+            g_free(last_state_name);
+        }
+        g_free(status_name);
     }
-    milter_statistics("[session][end][%s][%s][%g](%u)",
-                      last_state_name, statistics_status_name, elapsed, tag);
-    g_free(status_name);
-    g_free(state_name);
-    g_free(last_state_name);
 
     milter_manager_leader_quit(leader);
 }
 
+typedef struct _LeaderFinishData
+{
+    MilterManager *manager;
+    MilterClientContext *client_context;
+} LeaderFinishData;
+
 static void
 teardown_client_context_signals (MilterClientContext *context,
-                                 MilterManagerLeader *leader)
+                                 MilterManagerLeader *leader,
+                                 LeaderFinishData *finished_data)
 {
 #define DISCONNECT(name)                                                \
     g_signal_handlers_disconnect_by_func(context,                       \
@@ -492,46 +656,40 @@ teardown_client_context_signals (MilterClientContext *context,
 #undef DISCONNECT
     g_signal_handlers_disconnect_by_func(leader,
                                          G_CALLBACK(cb_leader_finished),
-                                         context);
-}
-
-static gboolean
-cb_idle_notify_finish_session_to_configuration (gpointer data)
-{
-    MilterManagerConfiguration *config = data;
-
-    milter_manager_configuration_session_finished(config);
-
-    return FALSE;
-}
-
-static gboolean
-cb_idle_unref_leader (gpointer data)
-{
-    MilterManagerLeader *leader = data;
-
-    g_object_unref(leader);
-
-    return FALSE;
+                                         finished_data);
 }
 
 static void
 cb_leader_finished (MilterFinishedEmittable *emittable, gpointer user_data)
 {
-    MilterClientContext *client_context = user_data;
-    GTimer *timer;
+    LeaderFinishData *finish_data = user_data;
+    MilterClientContext *client_context;
     MilterManagerLeader *leader;
-    MilterManagerConfiguration *config;
+    MilterManagerPrivate *priv;
 
-    timer = milter_client_context_get_private_data(client_context);
-    g_timer_stop(timer);
+    client_context = finish_data->client_context;
 
     leader = MILTER_MANAGER_LEADER(emittable);
-    teardown_client_context_signals(client_context, leader);
+    teardown_client_context_signals(client_context, leader, finish_data);
 
-    config = milter_manager_leader_get_configuration(leader);
-    g_idle_add(cb_idle_notify_finish_session_to_configuration, config);
-    g_idle_add(cb_idle_unref_leader, leader);
+    priv = MILTER_MANAGER_GET_PRIVATE(finish_data->manager);
+    if (!priv->connection_checking) {
+        GList *node;
+        node = g_list_find(priv->leaders, leader);
+        if (node) {
+            if (priv->next_connection_checked_leader == node)
+                priv->next_connection_checked_leader = g_list_next(node);
+            priv->leaders = g_list_delete_link(priv->leaders, node);
+        }
+        if (!priv->leaders) {
+            milter_debug("[manager][connection-check][dispose] no leaders");
+            dispose_periodical_connection_checker(priv);
+        }
+    }
+
+    priv->finished_leaders = g_list_prepend(priv->finished_leaders, leader);
+
+    g_free(finish_data);
 }
 
 static void
@@ -540,10 +698,12 @@ setup_context_signals (MilterClientContext *context,
 {
     MilterManagerLeader *leader;
     MilterManagerPrivate *priv;
+    LeaderFinishData *finish_data;
 
     priv = MILTER_MANAGER_GET_PRIVATE(manager);
 
     leader = milter_manager_leader_new(priv->configuration, context);
+    priv->leaders = g_list_prepend(priv->leaders, leader);
 
 #define CONNECT(name)                                   \
     g_signal_connect(context, #name,                    \
@@ -568,28 +728,31 @@ setup_context_signals (MilterClientContext *context,
     CONNECT(finished);
 
 #undef CONNECT
+
+    finish_data = g_new(LeaderFinishData, 1);
+    finish_data->manager = manager;
+    finish_data->client_context = context;
     g_signal_connect(leader, "finished",
-                     G_CALLBACK(cb_leader_finished), context);
+                     G_CALLBACK(cb_leader_finished), finish_data);
     milter_manager_leader_set_launcher_channel(leader,
                                                priv->launcher_read_channel,
                                                priv->launcher_write_channel);
+
+    g_signal_emit_by_name(priv->configuration, "connected", leader);
 }
 
 static void
 connection_established (MilterClient *client, MilterClientContext *context)
 {
-    GTimer *timer;
-    guint tag;
+    MilterManager *manager;
 
+    manager = MILTER_MANAGER(client);
     setup_context_signals(context, MILTER_MANAGER(client));
 
-    timer = g_timer_new();
-    milter_client_context_set_private_data(context, timer,
-                                           (GDestroyNotify)g_timer_destroy);
+    milter_debug("[%u] [manager][session][start]",
+                 milter_agent_get_tag(MILTER_AGENT(context)));
 
-    tag = milter_agent_get_tag(MILTER_AGENT(context));
-    milter_debug("[%u] [manager][session][start]", tag),
-    milter_statistics("[session][start](%u)", tag);
+    start_periodical_connection_checker(manager);
 }
 
 static gchar *
@@ -762,12 +925,67 @@ set_effective_group (MilterClient *client, const gchar *effective_group)
                                                      effective_group);
 }
 
+static guint
+get_maintenance_interval (MilterClient *client)
+{
+    MilterManager *manager;
+    MilterManagerPrivate *priv;
+    MilterManagerConfiguration *configuration;
+
+    manager = MILTER_MANAGER(client);
+    priv = MILTER_MANAGER_GET_PRIVATE(manager);
+    configuration = priv->configuration;
+    return milter_manager_configuration_get_maintenance_interval(configuration);
+}
+
+static void
+set_maintenance_interval (MilterClient *client, guint n_sessions)
+{
+    MilterManager *manager;
+    MilterManagerPrivate *priv;
+    MilterManagerConfiguration *configuration;
+
+    manager = MILTER_MANAGER(client);
+    priv = MILTER_MANAGER_GET_PRIVATE(manager);
+    configuration = priv->configuration;
+    milter_manager_configuration_set_maintenance_interval(configuration,
+                                                          n_sessions);
+}
+
+static void
+maintain (MilterClient *client)
+{
+    MilterManagerPrivate *priv;
+
+    milter_debug("[manager][maintain]");
+
+    priv = MILTER_MANAGER_GET_PRIVATE(client);
+    milter_manager_configuration_maintain(priv->configuration);
+}
+
+static void
+sessions_finished (MilterClient *client, guint n_finished_sessions)
+{
+    MilterManagerPrivate *priv;
+
+    milter_debug("[manager][sessions][finished] %u", n_finished_sessions);
+
+    priv = MILTER_MANAGER_GET_PRIVATE(client);
+    dispose_finished_leaders(priv);
+}
+
 MilterManagerConfiguration *
 milter_manager_get_configuration (MilterManager *manager)
 {
     return MILTER_MANAGER_GET_PRIVATE(manager)->configuration;
 }
 
+const GList *
+milter_manager_get_leaders (MilterManager *manager)
+{
+    return MILTER_MANAGER_GET_PRIVATE(manager)->leaders;
+}
+
 void
 milter_manager_set_launcher_channel (MilterManager *manager,
                                      GIOChannel *read_channel,
