@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -78,8 +78,11 @@ static gboolean real_load         (MilterManagerConfiguration *configuration,
 static gboolean real_load_custom  (MilterManagerConfiguration *configuration,
                                    const gchar                *file_name,
                                    GError                    **error);
-static void     real_maintain     (MilterManagerConfiguration *configuration);
+static gboolean real_maintain     (MilterManagerConfiguration *configuration,
+                                   GError                    **error);
 static gchar   *real_dump         (MilterManagerConfiguration *configuration);
+static gboolean real_clear        (MilterManagerConfiguration *configuration,
+                                   GError                    **error);
 
 static gpointer milter_manager_ruby_configuration_parent_class = NULL;
 static GType    milter_manager_ruby_configuration_type_id = 0;
@@ -107,6 +110,7 @@ milter_manager_ruby_configuration_class_init (MilterManagerRubyConfigurationClas
     configuration_class->load_custom = real_load_custom;
     configuration_class->maintain = real_maintain;
     configuration_class->dump = real_dump;
+    configuration_class->clear = real_clear;
 }
 
 static void
@@ -312,12 +316,17 @@ init_ruby (void)
 {
     int argc;
     char *argv[] = {"milter-manager"};
+    const gchar *milter_manager_rubylib;
 
     argc = sizeof(argv) / sizeof(*argv);
     ruby_init_without_signal_change();
     ruby_script(argv[0]);
     ruby_set_argv(argc, argv);
     rb_argv0 = rb_gv_get("$PROGRAM_NAME");
+    milter_manager_rubylib = g_getenv("MILTER_MANAGER_RUBYLIB");
+    if (milter_manager_rubylib) {
+        ruby_incpush(milter_manager_rubylib);
+    }
     ruby_incpush(BINDING_LIB_DIR);
     ruby_incpush(BINDING_EXT_DIR);
     ruby_init_loadpath();
@@ -411,27 +420,27 @@ get_property (GObject    *object,
 }
 
 static gboolean
-load (MilterManagerConfiguration *_configuration, ID method_name,
+load (MilterManagerConfiguration *_configuration, const gchar *method_name,
       const gchar *file_name, GError **error)
 {
     MilterManagerRubyConfiguration *configuration;
     GError *local_error = NULL;
-    gboolean success;
+    gboolean success = TRUE;
 
     configuration = MILTER_MANAGER_RUBY_CONFIGURATION(_configuration);
     rb_funcall_protect(&local_error,
                        rb_mMilterManagerConfigurationLoader,
-                       method_name, 2,
+                       rb_intern(method_name), 2,
                        GOBJ2RVAL(configuration),
                        rb_str_new2(file_name));
 
     if (local_error) {
         success = FALSE;
-        if (!error)
-            milter_error("%s", local_error->message);
+        if (!error) {
+            milter_error("[ruby-configuration][error][%s] <%s>: %s",
+                         method_name, file_name, local_error->message);
+        }
         g_propagate_error(error, local_error);
-    } else {
-        success = TRUE;
     }
 
     return success;
@@ -441,20 +450,40 @@ static gboolean
 real_load (MilterManagerConfiguration *_configuration, const gchar *file_name,
            GError **error)
 {
-    return load(_configuration, rb_intern("load"), file_name, error);
+    return load(_configuration, "load", file_name, error);
 }
 
 static gboolean
 real_load_custom (MilterManagerConfiguration *_configuration,
                   const gchar *file_name, GError **error)
 {
-    return load(_configuration, rb_intern("load_custom"), file_name, error);
+    return load(_configuration, "load_custom", file_name, error);
 }
 
-static void
-real_maintain (MilterManagerConfiguration *_configuration)
+static gboolean
+real_maintain (MilterManagerConfiguration *_configuration, GError **error)
 {
+    MilterManagerRubyConfiguration *configuration;
+    GError *local_error = NULL;
+    gboolean success = TRUE;
+
     rb_gc_start();
+
+    configuration = MILTER_MANAGER_RUBY_CONFIGURATION(_configuration);
+    rb_funcall_protect(&local_error,
+                       GOBJ2RVAL(configuration),
+                       rb_intern("maintained"),
+                       0);
+    if (local_error) {
+        success = FALSE;
+        if (!error) {
+            milter_error("[ruby-configuration][error][maintain] %s",
+                         local_error->message);
+        }
+        g_propagate_error(error, local_error);
+    }
+
+    return success;
 }
 
 static gchar *
@@ -477,6 +506,30 @@ real_dump (MilterManagerConfiguration *_configuration)
     }
 }
 
+static gboolean
+real_clear (MilterManagerConfiguration *_configuration, GError **error)
+{
+    MilterManagerRubyConfiguration *configuration;
+    GError *local_error = NULL;
+    gboolean success = TRUE;
+
+    configuration = MILTER_MANAGER_RUBY_CONFIGURATION(_configuration);
+    rb_funcall_protect(&local_error,
+                       GOBJ2RVAL(configuration),
+                       rb_intern("clear"),
+                       0);
+    if (local_error) {
+        success = FALSE;
+        if (!error) {
+            milter_error("[ruby-configuration][error][clear] %s",
+                         local_error->message);
+        }
+        g_propagate_error(error, local_error);
+    }
+
+    return success;
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
