@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008  Kouhei Sutou <kou@cozmixng.org>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -97,13 +97,14 @@ setup_signals (MilterClientContext *context)
 void
 setup (void)
 {
-    context = milter_client_context_new();
+    context = milter_client_context_new(NULL);
 
     channel = gcut_string_io_channel_new(NULL);
     g_io_channel_set_encoding(channel, NULL, NULL);
     writer = milter_writer_io_channel_new(channel);
 
     milter_agent_set_writer(MILTER_AGENT(context), writer);
+    milter_agent_start(MILTER_AGENT(context), NULL);
     setup_signals(context);
 
     command_encoder = MILTER_COMMAND_ENCODER(milter_command_encoder_new());
