@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -30,6 +30,7 @@
 #include <glib/gstdio.h>
 
 #include <milter/client.h>
+#include <milter/client/milter-client-private.h>
 #include <milter-test-utils.h>
 
 #include <gcutter.h>
@@ -47,7 +48,18 @@ void test_remove_unix_socket_on_create_accessor (void);
 void test_remove_unix_socket_on_create (void);
 void test_not_remove_unix_socket_on_create (void);
 void test_suspend_time_on_unacceptable (void);
+void test_max_connections (void);
+void test_effective_user (void);
+void test_effective_group (void);
+void test_maintenance_interval (void);
 void test_connection_spec (void);
+void test_need_maintain (void);
+void test_need_maintain_over (void);
+void test_need_maintain_over_interval (void);
+void test_need_maintain_never (void);
+void test_need_maintain_no_processing_sessions (void);
+void test_need_maintain_no_processing_sessions_below_processed_sessions (void);
+void test_need_maintain_no_processing_sessions_no_interval (void);
 
 static MilterClient *client;
 static MilterTestServer *server;
@@ -754,6 +766,122 @@ test_connection_spec (void)
                             milter_client_get_connection_spec(client));
 }
 
+void
+test_max_connections (void)
+{
+    cut_assert_equal_uint(
+        0, milter_client_get_max_connections(client));
+    milter_client_set_max_connections(client, 10);
+    cut_assert_equal_uint(
+        10, milter_client_get_max_connections(client));
+}
+
+void
+test_effective_user (void)
+{
+    cut_assert_equal_string(
+        NULL, milter_client_get_effective_user(client));
+    milter_client_set_effective_user(client, "nobody");
+    cut_assert_equal_string(
+        "nobody", milter_client_get_effective_user(client));
+}
+
+void
+test_effective_group (void)
+{
+    cut_assert_equal_string(
+        NULL, milter_client_get_effective_group(client));
+    milter_client_set_effective_group(client, "nogroup");
+    cut_assert_equal_string(
+        "nogroup", milter_client_get_effective_group(client));
+}
+
+void
+test_maintenance_interval (void)
+{
+    cut_assert_equal_uint(
+        0, milter_client_get_maintenance_interval(client));
+    milter_client_set_maintenance_interval(client, 10);
+    cut_assert_equal_uint(
+        10, milter_client_get_maintenance_interval(client));
+}
+
+void
+test_need_maintain (void)
+{
+    milter_client_set_maintenance_interval(client, 10);
+    milter_client_set_n_processing_sessions(client, 3);
+
+    milter_client_set_n_processed_sessions(client, 10);
+    cut_assert_false(milter_client_need_maintain(client, 0));
+    cut_assert_true(milter_client_need_maintain(client, 1));
+
+    milter_client_set_n_processed_sessions(client, 11);
+    cut_assert_false(milter_client_need_maintain(client, 1));
+    cut_assert_true(milter_client_need_maintain(client, 2));
+}
+
+void
+test_need_maintain_over (void)
+{
+    milter_client_set_maintenance_interval(client, 10);
+    milter_client_set_n_processing_sessions(client, 3);
+
+    milter_client_set_n_processed_sessions(client, 21);
+    cut_assert_false(milter_client_need_maintain(client, 1));
+    cut_assert_true(milter_client_need_maintain(client, 2));
+}
+
+void
+test_need_maintain_over_interval (void)
+{
+    milter_client_set_maintenance_interval(client, 2);
+    milter_client_set_n_processing_sessions(client, 3);
+
+    milter_client_set_n_processed_sessions(client, 4);
+    cut_assert_true(milter_client_need_maintain(client, 3));
+}
+
+void
+test_need_maintain_never (void)
+{
+    milter_client_set_maintenance_interval(client, 0);
+    milter_client_set_n_processing_sessions(client, 3);
+
+    cut_assert_false(milter_client_need_maintain(client, 0));
+    cut_assert_false(milter_client_need_maintain(client, 1));
+
+    milter_client_set_n_processed_sessions(client, 100);
+    cut_assert_false(milter_client_need_maintain(client, 1));
+}
+
+void
+test_need_maintain_no_processing_sessions (void)
+{
+    milter_client_set_maintenance_interval(client, 5);
+    milter_client_set_n_processing_sessions(client, 0);
+    cut_assert_false(milter_client_need_maintain(client, 0));
+    cut_assert_true(milter_client_need_maintain(client, 1));
+}
+
+void
+test_need_maintain_no_processing_sessions_below_processed_sessions (void)
+{
+    milter_client_set_maintenance_interval(client, 5);
+    milter_client_set_n_processed_sessions(client, 3);
+    cut_assert_false(milter_client_need_maintain(client, 0));
+    cut_assert_true(milter_client_need_maintain(client, 1));
+}
+
+void
+test_need_maintain_no_processing_sessions_no_interval (void)
+{
+    milter_client_set_maintenance_interval(client, 0);
+    milter_client_set_n_processing_sessions(client, 0);
+    cut_assert_false(milter_client_need_maintain(client, 0));
+    cut_assert_true(milter_client_need_maintain(client, 1));
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
