@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2009-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -255,7 +255,7 @@ setup (void)
 {
     smfi_register(smfilter);
 
-    client_context = milter_client_context_new();
+    client_context = milter_client_context_new(NULL);
     expected_output = NULL;
 
     context = smfi_context_new(client_context);
@@ -264,6 +264,7 @@ setup (void)
     g_io_channel_set_encoding(channel, NULL, NULL);
     writer = milter_writer_io_channel_new(channel);
     milter_agent_set_writer(MILTER_AGENT(client_context), writer);
+    milter_agent_start(MILTER_AGENT(client_context), NULL);
 
     command_encoder = MILTER_COMMAND_ENCODER(milter_command_encoder_new());
     reply_encoder = MILTER_REPLY_ENCODER(milter_reply_encoder_new());
