@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -112,6 +112,11 @@ static GList *test_clients;
 
 static MilterOption *actual_option;
 
+static struct sockaddr *actual_address;
+
+static MilterLogLevelFlags original_log_level;
+static gboolean default_handler_disconnected;
+
 void
 cut_startup (void)
 {
@@ -359,7 +364,7 @@ clear_n_emitted (void)
 }
 
 void
-setup (void)
+cut_setup (void)
 {
     scenario_dir = g_build_filename(milter_test_get_base_dir(),
                                     "fixtures",
@@ -394,6 +399,11 @@ setup (void)
     test_clients = NULL;
 
     actual_option = NULL;
+
+    actual_address = NULL;
+
+    original_log_level = milter_get_log_level();
+    default_handler_disconnected = FALSE;
 }
 
 static void
@@ -427,6 +437,10 @@ arguments_free (GArray *arguments)
 void
 cut_teardown (void)
 {
+    milter_set_log_level(original_log_level);
+    if (default_handler_disconnected)
+        milter_logger_connect_default_handler(milter_logger());
+
     if (config)
         g_object_unref(config);
     if (children)
@@ -474,6 +488,17 @@ cut_teardown (void)
 
     if (actual_option)
         g_object_unref(actual_option);
+
+    if (actual_address)
+        g_free(actual_address);
+}
+
+static void
+disconnect_default_handler (void)
+{
+    milter_set_log_level(MILTER_LOG_LEVEL_ALL);
+    milter_logger_disconnect_default_handler(milter_logger());
+    default_handler_disconnected = TRUE;
 }
 
 static void
@@ -594,7 +619,7 @@ wait_finished_helper (void)
     }
     g_source_remove(timeout_waiting_id);
 
-    cut_assert_true(timeout_waiting, "timeout");
+    cut_assert_true(timeout_waiting, cut_message("timeout"));
     cut_assert_operator_uint(n_finished_emitted, > , n_finished_emitted_before);
 }
 
@@ -616,7 +641,7 @@ wait_children_error_helper (void)
     }
     g_source_remove(timeout_waiting_id);
 
-    cut_assert_true(timeout_waiting, "timeout");
+    cut_assert_true(timeout_waiting, cut_message("timeout"));
     cut_assert_not_null(actual_error);
 }
 
@@ -694,7 +719,7 @@ setup_process_launcher (MilterManagerChildren *children)
     g_io_channel_unref(read_channel);
     g_io_channel_unref(write_channel);
 
-    milter_agent_start(MILTER_AGENT(launcher));
+    milter_agent_start(MILTER_AGENT(launcher), NULL);
 }
 
 static void
@@ -795,7 +820,8 @@ assert_response_error (MilterManagerTestScenario *scenario, const gchar *group)
 
         type_name = get_string(scenario, group, "error_type");
         type = g_type_from_name(type_name);
-        cut_assert_operator_uint(0, <, type, "name: <%s>", type_name);
+        cut_assert_operator_uint(0, <, type,
+                                 cut_message("name: <%s>", type_name));
         code = get_enum(scenario, group, "error_code", type);
 
         message = get_string(scenario, group, "error_message");
@@ -852,7 +878,7 @@ assert_response_common (MilterManagerTestScenario *scenario, const gchar *group)
 
     actual_n_emitted = get_n_emitted(response);
     cut_assert_equal_uint(n_emitted, actual_n_emitted,
-                          "[%s](%s)", group, response);
+                          cut_message("[%s](%s)", group, response));
 }
 
 static void
@@ -1158,6 +1184,7 @@ void
 test_connect (void)
 {
     struct sockaddr_in address;
+    socklen_t actual_address_length = 0;
     const gchar host_name[] = "mx.local.net";
     const gchar ip_address[] = "192.168.123.123";
 
@@ -1167,11 +1194,21 @@ test_connect (void)
     address.sin_port = g_htons(50443);
     inet_aton(ip_address, &(address.sin_addr));
 
+    cut_assert_false(milter_manager_children_get_smtp_client_address(
+                         children, &actual_address, &actual_address_length));
+    cut_assert_equal_memory(NULL, 0, actual_address, actual_address_length);
+
     milter_manager_children_connect(children,
                                     host_name,
                                     (struct sockaddr *)(&address),
                                     sizeof(address));
     wait_reply(1, n_continue_emitted);
+
+    cut_assert_true(milter_manager_children_get_smtp_client_address(
+                        children, &actual_address, &actual_address_length));
+    cut_assert_equal_uint(sizeof(address), actual_address_length);
+    cut_assert_equal_memory(&address, sizeof(address),
+                            actual_address, actual_address_length);
 }
 
 static gboolean
@@ -1606,6 +1643,7 @@ test_no_negotiation (void)
     MilterLogger *logger;
     MilterManagerChild *child10026, *child10027;
 
+    disconnect_default_handler();
     logger = milter_logger();
     log_signal_id = g_signal_connect(logger, "log", G_CALLBACK(cb_log), NULL);
 
@@ -1629,14 +1667,12 @@ test_no_negotiation (void)
                         "[%u] [children][error][connection] [%u] "
                         "Failed to connect to inet:10027@localhost: "
                         "Connection refused: milter@10027\n"
-                        "[%u] [children][error][negotiate][no-response] [%u] "
-                        "milter@10027\n",
+                        "[%u] [children][error][negotiate][no-response]\n",
                         milter_manager_children_get_tag(children),
                         milter_agent_get_tag(MILTER_AGENT(child10026)),
                         milter_manager_children_get_tag(children),
                         milter_agent_get_tag(MILTER_AGENT(child10027)),
-                        milter_manager_children_get_tag(children),
-                        milter_agent_get_tag(MILTER_AGENT(child10027))),
+                        milter_manager_children_get_tag(children)),
         error_message->str);
 }
 
@@ -1684,6 +1720,7 @@ test_end_of_message_timeout (void)
 
     cut_trace(test_end_of_header());
 
+    disconnect_default_handler();
     logger = milter_logger();
     log_signal_id = g_signal_connect(logger, "log", G_CALLBACK(cb_log), NULL);
 
@@ -1696,7 +1733,8 @@ test_end_of_message_timeout (void)
     wait_reply(1, n_end_of_message_timeout_emitted);
 
     cut_assert_equal_string(
-        cut_take_printf("[%u] [children][timeout][end-of-message] [%u] "
+        cut_take_printf("[%u] [children][timeout][end-of-message]"
+                        "[end-of-message][accept] [%u] "
                         "milter@10026\n",
                         milter_manager_children_get_tag(children),
                         milter_agent_get_tag(MILTER_AGENT(child))),
@@ -1716,6 +1754,7 @@ test_writing_timeout (void)
 
     cut_trace(test_negotiate());
 
+    disconnect_default_handler();
     logger = milter_logger();
     log_signal_id = g_signal_connect(logger, "log", G_CALLBACK(cb_log), NULL);
 
@@ -1728,6 +1767,7 @@ test_writing_timeout (void)
     g_io_channel_unref(channel);
     child = milter_manager_children_get_children(children)->next->data;
     milter_agent_set_writer(MILTER_AGENT(child), writer);
+    milter_writer_start(writer, NULL);
     g_object_unref(writer);
 
     prepare_timeout_test(child, channel);
@@ -1745,7 +1785,8 @@ test_writing_timeout (void)
     wait_reply(1, n_continue_emitted);
 
     cut_assert_equal_string(
-        cut_take_printf("[%u] [children][timeout][writing] [%u] milter@10027\n",
+        cut_take_printf("[%u] [children][timeout][writing][negotiate][accept] "
+                        "[%u] milter@10027\n",
                         milter_manager_children_get_tag(children),
                         milter_agent_get_tag(MILTER_AGENT(child))),
         error_message->str);
@@ -1768,6 +1809,7 @@ test_reading_timeout (void)
 
     cut_trace(test_negotiate());
 
+    disconnect_default_handler();
     logger = milter_logger();
     log_signal_id = g_signal_connect(logger, "log", G_CALLBACK(cb_log), NULL);
 
@@ -1787,7 +1829,8 @@ test_reading_timeout (void)
     wait_reply(1, n_reading_timeout_emitted);
 
     cut_assert_equal_string(
-        cut_take_printf("[%u] [children][timeout][reading] [%u] milter@10027\n",
+        cut_take_printf("[%u] [children][timeout][reading][connect][accept] "
+                        "[%u] milter@10027\n",
                         milter_manager_children_get_tag(children),
                         milter_agent_get_tag(MILTER_AGENT(child))),
         error_message->str);
