@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -50,6 +50,7 @@ void test_manager_connection_spec (void);
 void test_fallback_status (void);
 void test_package_platform (void);
 void test_package_options (void);
+void test_connection_check_interval (void);
 void test_egg (void);
 void test_find_egg (void);
 void test_remove_egg (void);
@@ -62,6 +63,7 @@ void test_load_absolute_path (void);
 void test_save_custom (void);
 void test_to_xml_full (void);
 void test_to_xml_signal (void);
+void test_location (void);
 
 static MilterManagerConfiguration *config;
 static MilterManagerEgg *egg;
@@ -163,7 +165,7 @@ child_equal (gconstpointer a, gconstpointer b)
 static void
 cb_attach_to (MilterManagerApplicableCondition *condition,
               MilterManagerChild *child, MilterManagerChildren *children,
-              gpointer user_data)
+              MilterClientContext *client_context, gpointer user_data)
 {
     gboolean *attached_to = user_data;
 
@@ -199,7 +201,8 @@ test_children (void)
     milter_manager_children_add_child(expected_children, child);
 
     cut_assert_false(attached_to);
-    actual_children = milter_manager_configuration_create_children(config);
+    actual_children = milter_manager_children_new(config);
+    milter_manager_configuration_setup_children(config, actual_children, NULL);
     milter_assert_equal_children(expected_children, actual_children);
     cut_assert_true(attached_to);
 }
@@ -340,11 +343,11 @@ void
 test_maintenance_interval (void)
 {
     cut_assert_equal_uint(
-        100,
+        10,
         milter_manager_configuration_get_maintenance_interval(config));
-    milter_manager_configuration_set_maintenance_interval(config, 1000);
+    milter_manager_configuration_set_maintenance_interval(config, 100);
     cut_assert_equal_uint(
-        1000,
+        100,
         milter_manager_configuration_get_maintenance_interval(config));
 }
 
@@ -486,6 +489,18 @@ test_package_options (void)
                             actual_package_options);
 }
 
+void
+test_connection_check_interval (void)
+{
+    cut_assert_equal_uint(
+        0,
+        milter_manager_configuration_get_connection_check_interval(config));
+    milter_manager_configuration_set_connection_check_interval(config, 5);
+    cut_assert_equal_uint(
+        5,
+        milter_manager_configuration_get_connection_check_interval(config));
+}
+
 static void
 milter_assert_default_configuration_helper (MilterManagerConfiguration *config)
 {
@@ -530,7 +545,7 @@ milter_assert_default_configuration_helper (MilterManagerConfiguration *config)
     cut_assert_false(milter_manager_configuration_is_daemon(config));
 
     cut_assert_equal_uint(
-        100,
+        10,
         milter_manager_configuration_get_maintenance_interval(config));
     cut_assert_equal_uint(
         MILTER_CLIENT_DEFAULT_SUSPEND_TIME_ON_UNACCEPTABLE,
@@ -554,12 +569,17 @@ milter_assert_default_configuration_helper (MilterManagerConfiguration *config)
         MILTER_MANAGER_PACKAGE_OPTIONS,
         milter_manager_configuration_get_package_options(config));
 
+    cut_assert_equal_uint(
+        0,
+        milter_manager_configuration_get_connection_check_interval(config));
+
     if (expected_children)
         g_object_unref(expected_children);
     expected_children = milter_manager_children_new(config);
     if (actual_children)
         g_object_unref(actual_children);
-    actual_children = milter_manager_configuration_create_children(config);
+    actual_children = milter_manager_children_new(config);
+    milter_manager_configuration_setup_children(config, actual_children, NULL);
     milter_assert_equal_children(expected_children, actual_children);
     g_object_unref(actual_children);
     actual_children = NULL;
@@ -739,9 +759,19 @@ test_remove_applicable_condition (void)
         milter_assert_default_configuration_helper(config),     \
         milter_assert_default_configuration(config))
 
+static gboolean
+cb_connected (MilterManagerConfiguration *configuration,
+              MilterManagerLeader *leader,
+              gpointer user_data)
+{
+    return TRUE;
+}
+
 void
 test_clear (void)
 {
+    gulong handler_id;
+
     milter_assert_default_configuration(config);
 
     test_privilege_mode();
@@ -758,9 +788,16 @@ test_clear (void)
     test_suspend_time_on_unacceptable();
     test_max_connections();
     test_max_file_descriptors();
+    test_connection_check_interval();
+
+    handler_id = g_signal_connect(config, "connected",
+                                  G_CALLBACK(cb_connected), NULL);
+    cut_assert_true(g_signal_handler_is_connected(config, handler_id));
 
     milter_manager_configuration_clear(config);
     milter_assert_default_configuration(config);
+
+    cut_assert_false(g_signal_handler_is_connected(config, handler_id));
 }
 
 void
@@ -934,6 +971,43 @@ test_to_xml_signal (void)
         actual_xml);
 }
 
+#define milter_assert_equal_location_keys(expected)             \
+    cut_trace_with_info_expression(                             \
+        milter_assert_equal_location_keys_helper(expected),     \
+        milter_assert_equal_location_keys(expected))
+
+static void
+milter_assert_equal_location_keys_helper (const GList *expected)
+{
+    GHashTable *locations;
+    GList *keys;
+
+    locations = milter_manager_configuration_get_locations(config);
+    keys = g_list_sort(g_hash_table_get_keys(locations), (GCompareFunc)strcmp);
+    gcut_assert_equal_list_string(expected, gcut_take_list(keys, NULL));
+}
+
+void
+test_location (void)
+{
+    const gchar key[] = "package.platform";
+    gconstpointer location;
+
+    milter_assert_equal_location_keys(NULL);
+
+    milter_manager_configuration_set_location(config, key,
+                                              "milter-manager.local.conf", 29);
+    milter_assert_equal_location_keys(gcut_take_new_list_string(key, NULL));
+    location = milter_manager_configuration_get_location(config, key);
+    cut_assert_equal_string("milter-manager.local.conf",
+                            g_dataset_get_data(location, "file"));
+    cut_assert_equal_int(29,
+                         GPOINTER_TO_INT(g_dataset_get_data(location, "line")));
+
+    milter_manager_configuration_reset_location(config, "package.platform");
+    milter_assert_equal_location_keys(NULL);
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab:sw=4
 */
