@@ -1,6 +1,6 @@
 /* -*- Mode: C; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
 /*
- *  Copyright (C) 2008-2009  Kouhei Sutou <kou@clear-code.com>
+ *  Copyright (C) 2008-2010  Kouhei Sutou <kou@clear-code.com>
  *
  *  This library is free software: you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License as published by
@@ -272,15 +272,15 @@ setup (void)
     reader = milter_reader_io_channel_new(channel);
     g_io_channel_unref(channel);
 
-    client_context = milter_client_context_new();
+    client_context = milter_client_context_new(NULL);
     milter_agent_set_writer(MILTER_AGENT(client_context), writer);
     g_object_unref(writer);
+    milter_agent_start(MILTER_AGENT(client_context), NULL);
 
     server = milter_manager_test_server_new();
     milter_agent_set_reader(MILTER_AGENT(server), reader);
     g_object_unref(reader);
-
-    milter_agent_start(MILTER_AGENT(server));
+    milter_agent_start(MILTER_AGENT(server), NULL);
 
     leader = milter_manager_leader_new(config, client_context);
     actual_error = NULL;
@@ -592,7 +592,8 @@ assert_response_error (MilterManagerTestScenario *scenario, const gchar *group)
 
         type_name = get_string(scenario, group, "error_type");
         type = g_type_from_name(type_name);
-        cut_assert_operator_uint(0, <, type, "name: <%s>", type_name);
+        cut_assert_operator_uint(0, <, type,
+                                 cut_message("name: <%s>", type_name));
         code = get_enum(scenario, group, "error_code", type);
 
         message = get_string(scenario, group, "error_message");
@@ -764,6 +765,23 @@ assert_macros_helper (MilterManagerTestScenario *scenario, const gchar *group,
     }
 }
 
+static gboolean
+option_equal (gconstpointer value1, gconstpointer value2)
+{
+    MilterOption *option1, *option2;
+
+    option1 = MILTER_OPTION(value1);
+    option2 = MILTER_OPTION(value2);
+
+    if (option1 == option2) {
+        return TRUE;
+    } else if (option1 == NULL || option2 == NULL) {
+        return FALSE;
+    } else {
+        return milter_option_equal(option1, option2);
+    }
+}
+
 static void
 do_negotiate (MilterManagerTestScenario *scenario, const gchar *group)
 {
@@ -790,7 +808,18 @@ do_negotiate (MilterManagerTestScenario *scenario, const gchar *group)
     gcut_assert_equal_list_object_custom(
         options,
         milter_manager_test_clients_collect_negotiate_options(started_clients),
-        (GEqualFunc)milter_option_equal);
+        option_equal);
+
+    if (milter_manager_test_scenario_has_group(main_scenario, group) &&
+        milter_manager_test_scenario_has_key(main_scenario, group, "option")) {
+        MilterManagerChildren *children;
+        options = get_option_list(main_scenario, group, "option");
+        children = milter_manager_leader_get_children(leader);
+        gcut_assert_equal_object_custom(
+            options->data,
+            milter_manager_children_get_option(children),
+            option_equal);
+    }
 }
 
 static void
@@ -833,7 +862,8 @@ do_connect (MilterManagerTestScenario *scenario, const gchar *group)
     assert_response(scenario, group);
     assert_alive(scenario, group);
 
-    expected_connect_infos = get_pair_list(scenario, group, "infos");
+    expected_connect_infos =
+        get_pair_list_without_sort(scenario, group, "infos");
     actual_connect_infos = collect_received_pairs(connect_host, connect_address);
     gcut_assert_equal_list(
         expected_connect_infos, actual_connect_infos,
@@ -861,7 +891,7 @@ do_helo (MilterManagerTestScenario *scenario, const gchar *group)
     expected_fqdns = get_string_g_list(scenario, group, "fqdns");
     gcut_assert_equal_list_string(expected_fqdns,
                                   collect_received_strings(helo_fqdn),
-                                  "[%s]", group);
+                                  cut_message("[%s]", group));
 }
 
 static void
@@ -882,7 +912,7 @@ do_envelope_from (MilterManagerTestScenario *scenario, const gchar *group)
     expected_froms = get_string_g_list(scenario, group, "froms");
     gcut_assert_equal_list_string(expected_froms,
                                   collect_received_strings(envelope_from),
-                                  "[%s]", group);
+                                  cut_message("[%s]", group));
 
     assert_macros(scenario, group, MILTER_COMMAND_ENVELOPE_FROM);
 }
@@ -905,7 +935,7 @@ do_envelope_recipient (MilterManagerTestScenario *scenario, const gchar *group)
     expected_recipients = get_string_g_list(scenario, group, "recipients");
     gcut_assert_equal_list_string(expected_recipients,
                                   collect_received_strings(envelope_recipient),
-                                  "[%s]", group);
+                                  cut_message("[%s]", group));
 }
 
 static void
@@ -1322,8 +1352,6 @@ data_scenario_basic (void)
 {
     cut_add_data("negotiate",
                  g_strdup("negotiate.txt"), g_free,
-                 "negotiate - failure",
-                 g_strdup("negotiate-failure.txt"), g_free,
                  "connect",
                  g_strdup("connect.txt"), g_free,
                  "connect - macro",
@@ -1437,6 +1465,32 @@ data_scenario_end_of_message_action (void)
                  NULL);
 }
 
+static void
+data_scenario_negotiate (void)
+{
+    cut_add_data("negotiate - no child",
+                 g_strdup("negotiate-no-child.txt"), g_free,
+                 "negotiate - no enabled child",
+                 g_strdup("negotiate-no-enabled-child.txt"), g_free,
+                 "negotiate - no available child",
+                 g_strdup("negotiate-no-available-child.txt"), g_free,
+                 NULL);
+}
+
+static void
+data_scenario_connect (void)
+{
+    cut_add_data("connect - no child",
+                 g_strdup("connect-no-child.txt"), g_free,
+                 "connect - no enabled child",
+                 g_strdup("connect-no-enabled-child.txt"), g_free,
+                 "connect - no available child",
+                 g_strdup("connect-no-available-child.txt"), g_free,
+                 "connect - policy",
+                 g_strdup("connect-policy.txt"), g_free,
+                 NULL);
+}
+
 static void
 data_scenario_helo (void)
 {
@@ -1614,11 +1668,20 @@ data_scenario_reply_code (void)
                  g_strdup("reply-code-invalid.txt"), g_free);
 }
 
+static void
+data_scenario_applicable_condition (void)
+{
+    cut_add_data("applicable condition - Remote Network",
+                 g_strdup("applicable-condition-remote-network.txt"), g_free);
+}
+
 void
 data_scenario (void)
 {
     data_scenario_basic();
     data_scenario_end_of_message_action();
+    data_scenario_negotiate();
+    data_scenario_connect();
     data_scenario_helo();
     data_scenario_envelope_from();
     data_scenario_envelope_recipient();
@@ -1628,6 +1691,7 @@ data_scenario (void)
     data_scenario_body();
     data_scenario_end_of_message();
     data_scenario_reply_code();
+    data_scenario_applicable_condition();
 }
 
 void
